# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from zope.deprecation import deprecation

from ziggurat_foundations.exc import ZigguratException
from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.resource import ResourceService
from ziggurat_foundations.models.services.resource_tree import \
    ResourceTreeService

__all__ = ['ResourceMixin']


class ResourceMixin(BaseModel):
    """
    Mixin for Resource model
    """

    __possible_permissions__ = ()

    _ziggurat_services = [ResourceService, ResourceTreeService]

    @declared_attr
    def __tablename__(self):
        return 'resources'

    @declared_attr
    def resource_id(self):
        return sa.Column(sa.Integer(), primary_key=True, nullable=False,
                         autoincrement=True)

    @declared_attr
    def parent_id(self):
        return sa.Column(sa.Integer(),
                         sa.ForeignKey('resources.resource_id',
                                       onupdate='CASCADE',
                                       ondelete='SET NULL'))

    @declared_attr
    def ordering(self):
        return sa.Column(sa.Integer(), default=0, nullable=False)

    @declared_attr
    def resource_name(self):
        return sa.Column(sa.Unicode(100), nullable=False)

    @declared_attr
    def resource_type(self):
        return sa.Column(sa.Unicode(30), nullable=False)

    @declared_attr
    def owner_group_id(self):
        return sa.Column(sa.Integer,
                         sa.ForeignKey('groups.id', onupdate='CASCADE',
                                       ondelete='SET NULL'), index=True)

    @declared_attr
    def owner_user_id(self):
        return sa.Column(sa.Integer,
                         sa.ForeignKey('users.id', onupdate='CASCADE',
                                       ondelete='SET NULL'), index=True)

    @declared_attr
    def group_permissions(self):
        """ returns all group permissions for this resource"""
        return sa.orm.relationship('GroupResourcePermission',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def user_permissions(self):
        """ returns all user permissions for this resource"""
        return sa.orm.relationship('UserResourcePermission',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def groups(self):
        """ returns all groups that have permissions for this resource"""
        return sa.orm.relationship('Group',
                                   secondary='groups_resources_permissions',
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def users(self):
        """ returns all users that have permissions for this resource"""
        return sa.orm.relationship('User',
                                   secondary='users_resources_permissions',
                                   passive_deletes=True,
                                   passive_updates=True)

    __mapper_args__ = {'polymorphic_on': resource_type}
    __table_args__ = {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'}

    def __repr__(self):
        return '<Resource: %s, %s, id: %s position: %s>' % (
            self.resource_type, self.resource_name, self.resource_id,
            self.ordering)

    @property
    def __acl__(self):
        raise ZigguratException("Model should implement __acl__")

    @sa.orm.validates('user_permissions', 'group_permissions')
    def validate_permission(self, key, permission):
        """ validate if resouce can have specific permission """
        if permission.perm_name not in self.__possible_permissions__:
            raise AssertionError('perm_name is not one of {}'.format(
                self.__possible_permissions__))
        return permission

    @deprecation.deprecate("Resource.perms_for_user "
                           "will be removed in 0.8, use service instead")
    def perms_for_user(self, user, db_session=None):
        """

        .. deprecated:: 0.8

        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return ResourceService.perms_for_user(
            self, user=user, db_session=db_session)

    @deprecation.deprecate("Resource.direct_perms_for_user "
                           "will be removed in 0.8, use service instead")
    def direct_perms_for_user(self, user, db_session=None):
        """

        .. deprecated:: 0.8

        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return ResourceService.direct_perms_for_user(
            self, user=user, db_session=db_session)

    @deprecation.deprecate("Resource.group_perms_for_user "
                           "will be removed in 0.8, use service instead")
    def group_perms_for_user(self, user, db_session=None):
        """

        .. deprecated:: 0.8

        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return ResourceService.group_perms_for_user(
            self, user=user, db_session=db_session)

    @deprecation.deprecate("Resource.users_for_perm "
                           "will be removed in 0.8, use service instead")
    def users_for_perm(self, perm_name, user_ids=None, group_ids=None,
                       limit_group_permissions=False, skip_group_perms=False,
                       db_session=None):
        """

        .. deprecated:: 0.8

        :param perm_name:
        :param user_ids:
        :param group_ids:
        :param limit_group_permissions:
        :param skip_group_perms:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return ResourceService.users_for_perm(
            self, perm_name, user_ids=user_ids, group_ids=group_ids,
            limit_group_permissions=limit_group_permissions,
            skip_group_perms=skip_group_perms, db_session=db_session)

    @classmethod
    @deprecation.deprecate("Resource.by_resource_id "
                           "will be removed in 0.8, use service instead")
    def by_resource_id(cls, resource_id, db_session=None):
        """

        .. deprecated:: 0.8

        :param resource_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return ResourceService.by_resource_id(resource_id=resource_id,
                                              db_session=db_session)

    @classmethod
    @deprecation.deprecate("Resource.perm_by_group_and_perm_name "
                           "will be removed in 0.8, use service instead")
    def perm_by_group_and_perm_name(cls, res_id, group_id, perm_name,
                                    db_session=None):
        """

        .. deprecated:: 0.8

        :param res_id:
        :param group_id:
        :param perm_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return ResourceService.perm_by_group_and_perm_name(
            resource_id=res_id, group_id=group_id, perm_name=perm_name,
            db_session=db_session)

    @deprecation.deprecate("Resource.groups_for_perm "
                           "will be removed in 0.8, use service instead")
    def groups_for_perm(self, perm_name, group_ids=None,
                        limit_group_permissions=False,
                        db_session=None):
        """

        .. deprecated:: 0.8

        :param perm_name:
        :param group_ids:
        :param limit_group_permissions:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return ResourceService.groups_for_perm(
            self, perm_name=perm_name, group_ids=group_ids,
            limit_group_permissions=limit_group_permissions,
            db_session=db_session)



"""normalize constraint and key names
correct keys for pre 0.5.6 naming convention

Revision ID: 438c27ec1c9
Revises: 439766f6104d
Create Date: 2015-06-13 21:16:32.358778

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '438c27ec1c9'
down_revision = '439766f6104d'

from alembic import op
import sqlalchemy as sa
from alembic.context import get_context
from sqlalchemy.dialects.postgresql.base import PGDialect
from sqlalchemy.engine.reflection import Inspector


# correct keys for pre 0.5.6 naming convention


def upgrade():
    c = get_context()
    insp = Inspector.from_engine(c.connection.engine)
    # existing migration
    # pre naming convention keys
    groups_permissions_pkey = 'groups_permissions_pkey'
    groups_pkey = 'groups_pkey'
    groups_resources_permissions_pkey = 'groups_resources_permissions_pkey'
    users_groups_pkey = 'users_groups_pkey'
    users_permissions_pkey = 'users_permissions_pkey'
    users_resources_permissions_pkey = 'users_resources_permissions_pkey'

    if isinstance(c.connection.engine.dialect, PGDialect):
        op.execute(
            'ALTER INDEX groups_unique_group_name_key RENAME to ix_groups_uq_group_name_key')

        op.drop_constraint('groups_permissions_perm_name_check',
                           'groups_permissions')
        op.execute('''
        ALTER TABLE groups_permissions
            ADD CONSTRAINT ck_groups_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        ''')

        op.drop_constraint('groups_resources_permissions_perm_name_check',
                           'groups_resources_permissions')
        op.execute('''
        ALTER TABLE groups_resources_permissions
              ADD CONSTRAINT ck_groups_resources_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        ''')

        op.drop_constraint('user_permissions_perm_name_check',
                           'users_permissions')
        op.execute('''
        ALTER TABLE users_permissions
          ADD CONSTRAINT ck_user_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        ''')

        op.drop_constraint('users_resources_permissions_perm_name_check',
                           'users_resources_permissions')
        op.execute('''
        ALTER TABLE users_resources_permissions
          ADD CONSTRAINT ck_users_resources_permissions_perm_name CHECK (perm_name::text = lower(perm_name::text));
        ''')

        op.execute(
            'ALTER INDEX users_email_key2 RENAME to ix_users_uq_lower_email')

        op.execute(
            'ALTER INDEX users_username_uq2 RENAME to ix_users_ux_lower_username')

        if groups_permissions_pkey == \
                insp.get_pk_constraint('groups_permissions')['name']:
            op.execute(
                'ALTER INDEX groups_permissions_pkey RENAME to pk_groups_permissions')

        if groups_pkey == insp.get_pk_constraint('groups')['name']:
            op.execute('ALTER INDEX groups_pkey RENAME to pk_groups')

        if groups_resources_permissions_pkey == \
                insp.get_pk_constraint('groups_resources_permissions')['name']:
            op.execute(
                'ALTER INDEX groups_resources_permissions_pkey RENAME to pk_groups_resources_permissions')

        if users_groups_pkey == insp.get_pk_constraint('users_groups')['name']:
            op.execute(
                'ALTER INDEX users_groups_pkey RENAME to pk_users_groups')

        if users_permissions_pkey == \
                insp.get_pk_constraint('users_permissions')['name']:
            op.execute(
                'ALTER INDEX users_permissions_pkey RENAME to pk_users_permissions')

        if users_resources_permissions_pkey == \
                insp.get_pk_constraint('users_resources_permissions')['name']:
            op.execute(
                'ALTER INDEX users_resources_permissions_pkey RENAME to pk_users_resources_permissions')

        if 'external_identities_pkey' == \
                insp.get_pk_constraint('external_identities')['name']:
            op.execute(
                'ALTER INDEX external_identities_pkey RENAME to pk_external_identities')

        if 'external_identities_local_user_name_fkey' in [c['name'] for c in
                                                          insp.get_foreign_keys(
                                                                  'external_identities')]:
            op.drop_constraint('external_identities_local_user_name_fkey',
                               'external_identities', type_='foreignkey')
            op.create_foreign_key(None, 'external_identities', 'users',
                                  remote_cols=['user_name'],
                                  local_cols=['local_user_name'],
                                  onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'groups_permissions_group_id_fkey' in [c['name'] for c in
                                                  insp.get_foreign_keys(
                                                          'groups_permissions')]:
            op.drop_constraint('groups_permissions_group_id_fkey',
                               'groups_permissions', type_='foreignkey')
            op.create_foreign_key(None, 'groups_permissions', 'groups',
                                  remote_cols=['id'],
                                  local_cols=['group_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'groups_group_name_key' in [c['name'] for c in
                                       insp.get_unique_constraints('groups')]:
            op.execute(
                'ALTER INDEX groups_group_name_key RENAME to uq_groups_group_name')

        if 'groups_resources_permissions_group_id_fkey' in [c['name'] for c in
                                                            insp.get_foreign_keys(
                                                                    'groups_resources_permissions')]:
            op.drop_constraint('groups_resources_permissions_group_id_fkey',
                               'groups_resources_permissions',
                               type_='foreignkey')
            op.create_foreign_key(None, 'groups_resources_permissions',
                                  'groups', remote_cols=['id'],
                                  local_cols=['group_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'groups_resources_permissions_resource_id_fkey' in [c['name'] for c
                                                               in
                                                               insp.get_foreign_keys(
                                                                       'groups_resources_permissions')]:
            op.drop_constraint('groups_resources_permissions_resource_id_fkey',
                               'groups_resources_permissions',
                               type_='foreignkey')
            op.create_foreign_key(None, 'groups_resources_permissions',
                                  'resources', remote_cols=['resource_id'],
                                  local_cols=['resource_id'],
                                  onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'resources_pkey' == insp.get_pk_constraint('resources')['name']:
            op.execute('ALTER INDEX resources_pkey RENAME to pk_resources')

        if 'resources_owner_group_id_fkey' in [c['name'] for c in
                                               insp.get_foreign_keys(
                                                       'resources')]:
            op.drop_constraint('resources_owner_group_id_fkey', 'resources',
                               type_='foreignkey')
            op.create_foreign_key(None, 'resources', 'groups',
                                  remote_cols=['id'],
                                  local_cols=['owner_group_id'],
                                  onupdate='CASCADE',
                                  ondelete='SET NULL')

        if 'resources_owner_user_id_fkey' in [c['name'] for c in
                                              insp.get_foreign_keys(
                                                      'resources')]:
            op.drop_constraint('resources_owner_user_id_fkey', 'resources',
                               type_='foreignkey')
            op.create_foreign_key(None, 'resources', 'users',
                                  remote_cols=['id'],
                                  local_cols=['owner_user_id'],
                                  onupdate='CASCADE',
                                  ondelete='SET NULL')

        if 'resources_parent_id_fkey' in [c['name'] for c in
                                          insp.get_foreign_keys('resources')]:
            op.drop_constraint('resources_parent_id_fkey', 'resources',
                               type_='foreignkey')
            op.create_foreign_key(None, 'resources', 'resources',
                                  remote_cols=['resource_id'],
                                  local_cols=['parent_id'], onupdate='CASCADE',
                                  ondelete='SET NULL')

        if 'users_pkey' == insp.get_pk_constraint('users')['name']:
            op.execute('ALTER INDEX users_pkey RENAME to pk_users')

        if 'users_email_key' in [c['name'] for c in
                                 insp.get_unique_constraints('users')]:
            op.execute('ALTER INDEX users_email_key RENAME to uq_users_email')

        if 'users_user_name_key' in [c['name'] for c in
                                     insp.get_unique_constraints('users')]:
            op.execute(
                'ALTER INDEX users_user_name_key RENAME to uq_users_user_name')

        if 'users_groups_group_id_fkey' in [c['name'] for c in
                                            insp.get_foreign_keys(
                                                    'users_groups')]:
            op.drop_constraint('users_groups_group_id_fkey', 'users_groups',
                               type_='foreignkey')
            op.create_foreign_key(None, 'users_groups', 'groups',
                                  remote_cols=['id'],
                                  local_cols=['group_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'users_groups_user_id_fkey' in [c['name'] for c in
                                           insp.get_foreign_keys(
                                                   'users_groups')]:
            op.drop_constraint('users_groups_user_id_fkey', 'users_groups',
                               type_='foreignkey')
            op.create_foreign_key(None, 'users_groups', 'users',
                                  remote_cols=['id'],
                                  local_cols=['user_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'users_permissions_user_id_fkey' in [c['name'] for c in
                                                insp.get_foreign_keys(
                                                        'users_permissions')]:
            op.drop_constraint('users_permissions_user_id_fkey',
                               'users_permissions', type_='foreignkey')
            op.create_foreign_key(None, 'users_permissions', 'users',
                                  remote_cols=['id'],
                                  local_cols=['user_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'users_resources_permissions_resource_id_fkey' in [c['name'] for c in
                                                              insp.get_foreign_keys(
                                                                      'users_resources_permissions')]:
            op.drop_constraint('users_resources_permissions_resource_id_fkey',
                               'users_resources_permissions',
                               type_='foreignkey')
            op.create_foreign_key(None, 'users_resources_permissions',
                                  'resources', remote_cols=['resource_id'],
                                  local_cols=['resource_id'],
                                  onupdate='CASCADE',
                                  ondelete='CASCADE')

        if 'users_resources_permissions_user_id_fkey' in [c['name'] for c in
                                                          insp.get_foreign_keys(
                                                                  'users_resources_permissions')]:
            op.drop_constraint('users_resources_permissions_user_id_fkey',
                               'users_resources_permissions',
                               type_='foreignkey')
            op.create_foreign_key(None, 'users_resources_permissions', 'users',
                                  remote_cols=['id'],
                                  local_cols=['user_id'], onupdate='CASCADE',
                                  ondelete='CASCADE')


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from zope.deprecation import deprecation

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.user import UserService

__all__ = ['UserMixin']


class UserMixin(BaseModel):
    """ Base mixin for User object representation.
        It supplies all the basic functionality from password hash generation
        and matching to utility methods used for querying database for users
        and their permissions or resources they have access to. It is meant
        to be extended with other application specific properties"""

    __mapper_args__ = {}
    __table_args__ = {'mysql_engine': 'InnoDB',
                      'mysql_charset': 'utf8'}

    _ziggurat_services = [UserService]

    @declared_attr
    def __tablename__(self):
        return 'users'

    @declared_attr
    def id(self):
        """ Unique identifier of user object"""
        return sa.Column(sa.Integer, primary_key=True, autoincrement=True)

    @declared_attr
    def user_name(self):
        """ Unique user name user object"""
        return sa.Column(sa.Unicode(128), unique=True)

    @declared_attr
    def user_password(self):
        """ Password hash for user object """
        return sa.Column(sa.Unicode(256))

    @declared_attr
    def email(self):
        """ Email for user object """
        return sa.Column(sa.Unicode(100), nullable=False, unique=True)

    @declared_attr
    def status(self):
        """ Status of user object """
        return sa.Column(sa.SmallInteger(), nullable=False, default=1)

    @declared_attr
    def security_code(self):
        """ Security code user object (can be used for password reset etc. """
        return sa.Column(sa.Unicode(256), default='default')

    @declared_attr
    def last_login_date(self):
        """ Date of user's last login """
        return sa.Column(sa.TIMESTAMP(timezone=False),
                         default=lambda x: datetime.utcnow(),
                         server_default=sa.func.now())

    @declared_attr
    def registered_date(self):
        """ Date of user's registration """
        return sa.Column(sa.TIMESTAMP(timezone=False),
                         default=lambda x: datetime.utcnow(),
                         server_default=sa.func.now())

    @declared_attr
    def security_code_date(self):
        """ Date of user's security code update """
        return sa.Column(sa.TIMESTAMP(timezone=False),
                         default=datetime(2000, 1, 1),
                         server_default='2000-01-01 01:01')

    def __repr__(self):
        return '<User: %s>' % self.user_name

    @declared_attr
    def groups_dynamic(self):
        """ returns dynamic relationship for groups - allowing for
        filtering of data """
        return sa.orm.relationship('Group', secondary='users_groups',
                                   lazy='dynamic',
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def user_permissions(self):
        """
        returns all direct non-resource permissions for this user,
        allows to assign new permissions to user::

            user.user_permissions.append(resource)
        """
        return sa.orm.relationship('UserPermission',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def resource_permissions(self):
        """ returns all direct resource permissions for this user """
        return sa.orm.relationship('UserResourcePermission',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def resources(cls):
        """ Returns all resources directly owned by user, can be used to assign
        ownership of new resources::

            user.resources.append(resource) """
        return sa.orm.relationship('Resource',
                                   cascade="all",
                                   passive_deletes=True,
                                   passive_updates=True,
                                   backref='owner',
                                   lazy='dynamic')

    @declared_attr
    def external_identities(self):
        """ dynamic relation for external identities for this user -
        allowing for filtering of data """
        return sa.orm.relationship('ExternalIdentity',
                                   lazy='dynamic',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True,
                                   backref='owner')

    @property
    @deprecation.deprecate("User.permissions will be removed in 0.8, "
                           "use service instead")
    def permissions(self):
        """

        .. deprecated:: 0.8

        :return:
        """
        db_session = get_db_session(None, self)
        return UserService.permissions(self, db_session=db_session)

    @deprecation.deprecate("User.resources_with_perms will be removed in 0.8, "
                           "use service instead")
    def resources_with_perms(self, perms, resource_ids=None,
                             resource_types=None,
                             db_session=None):
        """

        .. deprecated:: 0.8

        :param perms:
        :param resource_ids:
        :param resource_types:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return UserService.resources_with_perms(
            self, perms=perms, resource_ids=resource_ids,
            resource_types=resource_types, db_session=db_session)

    @deprecation.deprecate("User.groups_with_resources will be removed in 0.8, "
                           "use service instead")
    def groups_with_resources(self):
        """

        .. deprecated:: 0.8

        :return:
        """
        return UserService.groups_with_resources(self)

    @deprecation.deprecate("User.resources_with_possible_perms "
                           "will be removed in 0.8, use service instead")
    def resources_with_possible_perms(self, resource_ids=None,
                                      resource_types=None,
                                      db_session=None):
        """

        .. deprecated:: 0.8

        :param resource_ids:
        :param resource_types:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return UserService.resources_with_possible_perms(
            self, resource_ids=resource_ids, resource_types=resource_types,
            db_session=db_session)

    @deprecation.deprecate("User.gravatar_url "
                           "will be removed in 0.8, use service instead")
    def gravatar_url(self, default='mm', **kwargs):
        """

        .. deprecated:: 0.8

        :param default:
        :param kwargs:
        :return:
        """
        return UserService.gravatar_url(self, default, **kwargs)

    @deprecation.deprecate("User.set_password "
                           "will be removed in 0.8, use service instead")
    def set_password(self, raw_password):
        """

        .. deprecated:: 0.8

        :param raw_password:
        :return:
        """
        return UserService.set_password(self, raw_password=raw_password)

    @deprecation.deprecate("User.check_password "
                           "will be removed in 0.8, use service instead")
    def check_password(self, raw_password):
        """

        .. deprecated:: 0.8

        :param raw_password:
        :return:
        """
        return UserService.check_password(self, raw_password=raw_password)

    @classmethod
    @deprecation.deprecate("User.migrate_password "
                           "will be removed in 0.8, use service instead")
    def migrate_password(self, raw_password):
        """

        .. deprecated:: 0.8

        :param raw_password:
        :return:
        """
        pass

    @classmethod
    @deprecation.deprecate("User.generate_random_pass "
                           "will be removed in 0.8, use service instead")
    def generate_random_pass(cls, chars=7):
        """

        .. deprecated:: 0.8

        :param chars:
        :return:
        """
        return UserService.generate_random_pass(chars=chars)

    @deprecation.deprecate("User.regenerate_security_code "
                           "will be removed in 0.8, use service instead")
    def regenerate_security_code(self):
        """

        .. deprecated:: 0.8

        :return:
        """
        return UserService.regenerate_security_code(self)

    @staticmethod
    @deprecation.deprecate("User.generate_random_string "
                           "will be removed in 0.8, use service instead")
    def generate_random_string(chars=7):
        """

        .. deprecated:: 0.8

        :param chars:
        :return:
        """
        return UserService.generate_random_pass(chars=chars)

    @classmethod
    @deprecation.deprecate("User.by_id "
                           "will be removed in 0.8, use service instead")
    def by_id(cls, user_id, db_session=None):
        """

        .. deprecated:: 0.8

        :param user_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_id(user_id=user_id, db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.by_user_name "
                           "will be removed in 0.8, use service instead")
    def by_user_name(cls, user_name, db_session=None):
        """

        .. deprecated:: 0.8

        :param user_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_user_name(user_name=user_name,
                                        db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.by_user_name_and_security_code "
                           "will be removed in 0.8, use service instead")
    def by_user_name_and_security_code(cls, user_name, security_code,
                                       db_session=None):
        """

        .. deprecated:: 0.8

        :param user_name:
        :param security_code:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_user_name_and_security_code(
            user_name=user_name, security_code=security_code,
            db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.by_user_names "
                           "will be removed in 0.8, use service instead")
    def by_user_names(cls, user_names, db_session=None):
        """

        .. deprecated:: 0.8

        :param user_names:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_user_names(user_names=user_names,
                                         db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.user_names_like "
                           "will be removed in 0.8, use service instead")
    def user_names_like(cls, user_name, db_session=None):
        """

        .. deprecated:: 0.8

        :param user_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.user_names_like(user_name=user_name,
                                           db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.by_email "
                           "will be removed in 0.8, use service instead")
    def by_email(cls, email, db_session=None):
        """

        .. deprecated:: 0.8

        :param email:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_email(email=email,
                                    db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.by_email_and_username "
                           "will be removed in 0.8, use service instead")
    def by_email_and_username(cls, email, user_name, db_session=None):
        """

        .. deprecated:: 0.8

        :param email:
        :param user_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.by_email_and_username(email=email,
                                                 user_name=user_name,
                                                 db_session=db_session)

    @classmethod
    @deprecation.deprecate("User.users_for_perms "
                           "will be removed in 0.8, use service instead")
    def users_for_perms(cls, perm_names, db_session=None):
        """

        .. deprecated:: 0.8

        :param perm_names:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserService.users_for_perms(perm_names=perm_names,
                                           db_session=db_session)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from zope.deprecation import deprecation

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.external_identity import \
    ExternalIdentityService

__all__ = ['ExternalIdentityMixin']


class ExternalIdentityMixin(BaseModel):
    """
    Mixin for External Identity model - it represents oAuth(or other) accounts
    attached to your user object
    """

    __table_args__ = (
        sa.PrimaryKeyConstraint('external_id', 'local_user_id', 'provider_name',
                                name='pk_external_identities'),
        {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8',
         })

    _ziggurat_services = [ExternalIdentityService]

    @declared_attr
    def __tablename__(self):
        return 'external_identities'

    @declared_attr
    def external_id(self):
        return sa.Column(sa.Unicode(255), default=u'', primary_key=True)

    @declared_attr
    def external_user_name(self):
        return sa.Column(sa.Unicode(255), default=u'')

    @declared_attr
    def local_user_id(self):
        return sa.Column(sa.Integer, sa.ForeignKey('users.id',
                                                   onupdate='CASCADE',
                                                   ondelete='CASCADE'),
                         primary_key=True)

    @declared_attr
    def provider_name(self):
        return sa.Column(sa.Unicode(50), default=u'', primary_key=True)

    @declared_attr
    def access_token(self):
        return sa.Column(sa.Unicode(512), default=u'')

    @declared_attr
    def alt_token(self):
        return sa.Column(sa.Unicode(512), default=u'')

    @declared_attr
    def token_secret(self):
        return sa.Column(sa.Unicode(512), default=u'')

    @classmethod
    @deprecation.deprecate("ExternalIdentity.by_external_id_and_provider "
                           "will be removed in 0.8, use service instead")
    def by_external_id_and_provider(cls, external_id, provider_name,
                                    db_session=None):
        """
        Backwards compatible alias to
        :class:`ziggurat_foundations.models.services.external_identity.ExternalIdentityService.by_external_id_and_provider`

        .. deprecated:: 0.8

        :param external_id:
        :param provider_name:
        :param db_session:
        :return: ExternalIdentity
        """
        db_session = get_db_session(db_session)
        return ExternalIdentityService.by_external_id_and_provider(
            external_id=external_id, provider_name=provider_name,
            db_session=db_session)

    @classmethod
    @deprecation.deprecate("ExternalIdentity.user_by_external_id_and_provider "
                           "will be removed in 0.8, use service instead")
    def user_by_external_id_and_provider(cls, external_id, provider_name,
                                         db_session=None):
        """
        Backwards compatible alias to
        :class:`ziggurat_foundations.models.services.external_identity.ExternalIdentityService.user_by_external_id_and_provider`

        .. deprecated:: 0.8

        :param external_id:
        :param provider_name:
        :param db_session:
        :return: User
        """
        db_session = get_db_session(db_session)
        return ExternalIdentityService.user_by_external_id_and_provider(
            external_id=external_id, provider_name=provider_name,
            db_session=db_session)



"""add id/parent id to resource structure

Revision ID: 5c84d7260c5
Revises: 24ab8d11f014
Create Date: 2011-11-11 00:09:09.624704

"""
from __future__ import unicode_literals

# downgrade revision identifier, used by Alembic.
revision = '5c84d7260c5'
down_revision = '24ab8d11f014'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'resources', sa.Column('parent_id', sa.BigInteger(),
                               sa.ForeignKey('resources.resource_id',
                                             onupdate='CASCADE',
                                             ondelete='SET NULL')))


def downgrade():
    pass



"""change all linking keys from chars to id's

Revision ID: 20671b28c538
Revises: 4c10d97c509
Create Date: 2012-07-07 21:49:21.906150

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '20671b28c538'
down_revision = '4c10d97c509'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects.mysql.base import MySQLDialect
from alembic.context import get_context
from sqlalchemy.engine.reflection import Inspector


def upgrade():
    c = get_context()
    insp = Inspector.from_engine(c.connection.engine)

    # existing migration
    # pre naming convention keys
    groups_permissions_pkey = 'groups_permissions_pkey'
    groups_pkey = 'groups_pkey'
    groups_resources_permissions_pkey = 'groups_resources_permissions_pkey'
    users_groups_pkey = 'users_groups_pkey'
    users_permissions_pkey = 'users_permissions_pkey'
    users_resources_permissions_pkey = 'users_resources_permissions_pkey'

    # inspected keys
    groups_permissions_pkey = insp.get_pk_constraint('groups_permissions')[
        'name']
    groups_pkey = insp.get_pk_constraint('groups')['name']
    groups_resources_permissions_pkey = \
    insp.get_pk_constraint('groups_resources_permissions')['name']
    users_groups_pkey = insp.get_pk_constraint('users_groups')['name']
    users_permissions_pkey = insp.get_pk_constraint('users_permissions')['name']
    users_resources_permissions_pkey = \
    insp.get_pk_constraint('users_resources_permissions')['name']

    op.drop_constraint('groups_pkey', 'groups', type_='primary')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('groups', sa.Column('id', sa.Integer, primary_key=True,
                                          autoincrement=False))
        op.create_primary_key(groups_pkey, 'groups', cols=['id'])
        op.alter_column('groups', 'id', type_=sa.Integer,
                        existing_type=sa.Integer, autoincrement=True,
                        existing_autoincrement=False, nullable=False)
    else:
        op.add_column('groups', sa.Column('id', sa.Integer, primary_key=True,
                                          autoincrement=True))
        op.create_primary_key(groups_pkey, 'groups', cols=['id'])

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        for t in ['groups_permissions', 'groups_resources_permissions',
                  'users_groups']:
            for constraint in insp.get_foreign_keys(t):
                if constraint['referred_columns'] == ['group_name']:
                    op.drop_constraint(constraint['name'], t,
                                       type_='foreignkey')

        for t in ['users_resources_permissions', 'users_permissions',
                  'users_groups']:
            for constraint in insp.get_foreign_keys(t):
                if constraint['referred_columns'] == ['user_name']:
                    op.drop_constraint(constraint['name'], t,
                                       type_='foreignkey')

        for constraint in insp.get_foreign_keys('resources'):
            if constraint['referred_columns'] in [['user_name'],
                                                  ['group_name']]:
                op.drop_constraint(constraint['name'], 'resources',
                                   type_='foreignkey')

    op.add_column('resources', sa.Column('owner_user_id', sa.Integer(),
                                         sa.ForeignKey('users.id',
                                                       onupdate='CASCADE',
                                                       ondelete='SET NULL')))
    op.add_column('resources', sa.Column('owner_group_id', sa.Integer(),
                                         sa.ForeignKey('groups.id',
                                                       onupdate='CASCADE',
                                                       ondelete='SET NULL')))
    # update the data
    op.execute('''update resources set owner_user_id = 
                (select id from users where users.user_name=owner_user_name)''')
    op.execute('''update resources set owner_group_id = 
                (select id from users where users.user_name=owner_group_name)''')

    # mysql is stupid as usual so we cant create FKEY and add PKEY later,
    # need to set PKEY first and then set FKEY
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('groups_permissions', sa.Column('group_id', sa.Integer()))
    else:
        op.add_column('groups_permissions', sa.Column('group_id', sa.Integer(),
                                                      sa.ForeignKey('groups.id',
                                                                    onupdate='CASCADE',
                                                                    ondelete='CASCADE')))

    op.execute('''update groups_permissions set group_id = 
    (select id from groups where groups.group_name=groups_permissions.group_name)''')

    op.drop_constraint(groups_permissions_pkey, 'groups_permissions',
                       type_='primary')
    op.create_primary_key(groups_permissions_pkey, 'groups_permissions',
                          cols=['group_id', 'perm_name'])
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'groups_permissions', 'groups',
                              remote_cols=['id'],
                              local_cols=['group_id'], onupdate='CASCADE',
                              ondelete='CASCADE')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('groups_resources_permissions',
                      sa.Column('group_id', sa.Integer()))
    else:
        op.add_column('groups_resources_permissions',
                      sa.Column('group_id', sa.Integer(),
                                sa.ForeignKey('groups.id', onupdate='CASCADE',
                                              ondelete='CASCADE')))

    op.execute('''update groups_resources_permissions set group_id = 
    (select id from groups where groups.group_name=groups_resources_permissions.group_name)''')
    op.drop_constraint(groups_resources_permissions_pkey,
                       'groups_resources_permissions',
                       type_='primary')
    op.create_primary_key(groups_resources_permissions_pkey,
                          'groups_resources_permissions',
                          cols=['group_id', 'resource_id', 'perm_name'])

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'groups_resources_permissions', 'groups',
                              remote_cols=['id'],
                              local_cols=['group_id'], onupdate='CASCADE',
                              ondelete='CASCADE')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('users_groups', sa.Column('group_id', sa.Integer()))
    else:
        op.add_column('users_groups', sa.Column('group_id', sa.Integer(),
                                                sa.ForeignKey('groups.id',
                                                              onupdate='CASCADE',
                                                              ondelete='CASCADE')))
    op.execute('''update users_groups set group_id = 
    (select id from groups where groups.group_name=users_groups.group_name)''')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('users_groups', sa.Column('user_id', sa.Integer()))
    else:
        op.add_column('users_groups', sa.Column('user_id', sa.Integer(),
                                                sa.ForeignKey('users.id',
                                                              onupdate='CASCADE',
                                                              ondelete='CASCADE')))
    op.execute('''update users_groups set user_id = 
    (select id from users where users.user_name=users_groups.user_name)''')
    op.drop_constraint(users_groups_pkey, 'users_groups', type='primary')
    op.create_primary_key(users_groups_pkey, 'users_groups',
                          cols=['user_id', 'group_id'])
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'users_groups', 'groups',
                              remote_cols=['id'],
                              local_cols=['group_id'], onupdate='CASCADE',
                              ondelete='CASCADE')
        op.create_foreign_key(None, 'users_groups', 'users',
                              remote_cols=['id'],
                              local_cols=['user_id'], onupdate='CASCADE',
                              ondelete='CASCADE')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('users_permissions', sa.Column('user_id', sa.Integer()))
    else:
        op.add_column('users_permissions', sa.Column('user_id', sa.Integer(),
                                                     sa.ForeignKey('users.id',
                                                                   onupdate='CASCADE',
                                                                   ondelete='CASCADE')))
    op.execute('''update users_permissions set user_id = 
    (select id from groups where groups.group_name=users_permissions.user_name)''')
    op.drop_constraint(users_permissions_pkey, 'users_permissions',
                       type='primary')
    op.create_primary_key(users_permissions_pkey, 'users_permissions',
                          cols=['user_id', 'perm_name'])
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'users_permissions', 'users',
                              remote_cols=['id'],
                              local_cols=['user_id'], onupdate='CASCADE',
                              ondelete='CASCADE')

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.add_column('users_resources_permissions',
                      sa.Column('user_id', sa.Integer()))
    else:
        op.add_column('users_resources_permissions',
                      sa.Column('user_id', sa.Integer(),
                                sa.ForeignKey('users.id', onupdate='CASCADE',
                                              ondelete='CASCADE')))

    op.execute('''update users_resources_permissions set user_id = 
    (select id from users where users.user_name=users_resources_permissions.user_name)''')
    op.drop_constraint(users_resources_permissions_pkey,
                       'users_resources_permissions',
                       type='primary')
    op.create_primary_key(users_resources_permissions_pkey,
                          'users_resources_permissions',
                          cols=['user_id', 'resource_id', 'perm_name'])
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'users_resources_permissions', 'users',
                              remote_cols=['id'],
                              local_cols=['user_id'], onupdate='CASCADE',
                              ondelete='CASCADE')

    op.drop_column('resources', 'owner_user_name')
    op.drop_column('resources', 'owner_group_name')
    op.drop_column('groups_permissions', 'group_name')
    op.drop_column('groups_resources_permissions', 'group_name')
    op.drop_column('users_resources_permissions', 'user_name')
    op.drop_column('users_groups', 'group_name')
    op.drop_column('users_groups', 'user_name')
    op.drop_column('users_permissions', 'user_name')


def downgrade():
    pass



"""groups pkey change

Revision ID: 3cfc41c4a5f0
Revises: 53927300c277
Create Date: 2012-06-27 02:15:58.776223

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '3cfc41c4a5f0'
down_revision = '53927300c277'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects.mysql.base import MySQLDialect
from alembic.context import get_context
from sqlalchemy.engine.reflection import Inspector


def upgrade():
    c = get_context()
    if isinstance(c.connection.engine.dialect, MySQLDialect):
        insp = Inspector.from_engine(c.connection.engine)
        for t in ['groups_permissions', 'groups_resources_permissions',
                  'users_groups', 'resources']:
            for constraint in insp.get_foreign_keys(t):
                if constraint['referred_columns'] == ['group_name']:
                    op.drop_constraint(constraint['name'], t,
                                       type='foreignkey')

    op.drop_column('groups', 'id')
    op.alter_column('groups', 'group_name',
                    type_=sa.Unicode(128),
                    existing_type=sa.Unicode(50),
                    )
    op.create_primary_key('groups_pkey', 'groups', cols=['group_name'])

    if isinstance(c.connection.engine.dialect, MySQLDialect):
        op.create_foreign_key(None, 'groups_permissions', 'groups',
                              remote_cols=['group_name'],
                              local_cols=['group_name'], onupdate='CASCADE',
                              ondelete='CASCADE')
        op.create_foreign_key(None, 'groups_resources_permissions', 'groups',
                              remote_cols=['group_name'],
                              local_cols=['group_name'], onupdate='CASCADE',
                              ondelete='CASCADE')
        op.create_foreign_key(None, 'users_groups', 'groups',
                              remote_cols=['group_name'],
                              local_cols=['group_name'], onupdate='CASCADE',
                              ondelete='CASCADE')
        op.create_foreign_key(None, 'resources', 'groups',
                              remote_cols=['group_name'],
                              local_cols=['owner_group_name'],
                              onupdate='CASCADE',
                              ondelete='SET NULL')


def downgrade():
    pass



# -*- coding: utf-8 -*-

from collections import namedtuple

import sqlalchemy as sa

from .models.base import get_db_session

try:
    from pyramid.security import Allow, Deny, ALL_PERMISSIONS

except ImportError as e:
    Allow = 'Allow'
    Deny = 'Deny'


    # borrowed directly from pyramid - to avoid dependency on pyramid itself
    # source https://github.com/Pylons/pyramid/blob/master/pyramid/security.py

    class AllPermissionsList(object):

        """ Stand in 'permission list' to represent all permissions """

        def __iter__(self):
            return ()

        def __contains__(self, other):
            return True

        def __eq__(self, other):
            return isinstance(other, self.__class__)


    ALL_PERMISSIONS = AllPermissionsList()

__all__ = ['ANY_PERMISSION_CLS', 'ANY_PERMISSION',
           'resource_permissions_for_users',
           'permission_to_04_acls', 'permission_to_pyramid_acls']


class ANY_PERMISSION_CLS(object):
    def __eq__(self, other):
        return '__any_permission__' == other

    def __ne__(self, other):
        return '__any_permission__' != other


ANY_PERMISSION = ANY_PERMISSION_CLS()

PermissionTuple = namedtuple('PermissionTuple',
                             ['user', 'perm_name', 'type', 'group', 'resource',
                              'owner', 'allowed'])


def resource_permissions_for_users(models_proxy, perm_names, resource_ids=None,
                                   user_ids=None, group_ids=None,
                                   resource_types=None,
                                   limit_group_permissions=False,
                                   skip_user_perms=False,
                                   skip_group_perms=False,
                                   db_session=None):
    """
    Returns permission tuples that match one of passed permission names
    perm_names - list of permissions that can be matched
    user_ids - restrict to specific users
    group_ids - restrict to specific groups
    resource_ids - restrict to specific resources
    limit_group_permissions - should be used if we do not want to have
    user objects returned for group permissions, this might cause performance
    issues for big groups
    """
    db_session = get_db_session(db_session)

    # fetch groups and their permissions (possibly with users belonging
    # to group if needed)
    query = db_session.query(models_proxy.GroupResourcePermission.perm_name,
                             models_proxy.User,
                             models_proxy.Group,
                             sa.literal('group').label('type'),
                             models_proxy.Resource)

    query = query.join(models_proxy.Group,
                       models_proxy.Group.id ==
                       models_proxy.GroupResourcePermission.group_id)
    query = query.join(models_proxy.Resource,
                       models_proxy.Resource.resource_id ==
                       models_proxy.GroupResourcePermission.resource_id)
    if limit_group_permissions:
        query = query.outerjoin(models_proxy.User,
                                models_proxy.User.id == None)
    else:
        query = query.join(models_proxy.UserGroup,
                           models_proxy.UserGroup.group_id ==
                           models_proxy.GroupResourcePermission.group_id)

        query = query.outerjoin(models_proxy.User,
                                models_proxy.User.id ==
                                models_proxy.UserGroup.user_id)

    if resource_ids:
        query = query.filter(
            models_proxy.GroupResourcePermission.resource_id.in_(resource_ids))
    if resource_types:
        query = query.filter(
            models_proxy.Resource.resource_type.in_(resource_types))

    if (perm_names not in ([ANY_PERMISSION], ANY_PERMISSION) and perm_names):
        query = query.filter(
            models_proxy.GroupResourcePermission.perm_name.in_(perm_names))
    if group_ids:
        query = query.filter(
            models_proxy.GroupResourcePermission.group_id.in_(group_ids))

    if user_ids and not limit_group_permissions:
        query = query.filter(
            models_proxy.UserGroup.user_id.in_(user_ids))

    # 2nd query that will fetch users with direct resource permissions
    query2 = db_session.query(models_proxy.UserResourcePermission.perm_name,
                              models_proxy.User,
                              models_proxy.Group,
                              sa.literal('user').label('type'),
                              models_proxy.Resource)
    query2 = query2.join(models_proxy.User,
                         models_proxy.User.id ==
                         models_proxy.UserResourcePermission.user_id)
    query2 = query2.join(models_proxy.Resource,
                         models_proxy.Resource.resource_id ==
                         models_proxy.UserResourcePermission.resource_id)

    # group needs to be present to work for union, but never actually matched
    query2 = query2.outerjoin(models_proxy.Group,
                              models_proxy.Group.id == None)
    if (perm_names not in ([ANY_PERMISSION], ANY_PERMISSION) and perm_names):
        query2 = query2.filter(
            models_proxy.UserResourcePermission.perm_name.in_(perm_names))
    if resource_ids:
        query2 = query2.filter(
            models_proxy.UserResourcePermission.resource_id.in_(resource_ids))
    if resource_types:
        query2 = query2.filter(
            models_proxy.Resource.resource_type.in_(resource_types))
    if user_ids:
        query2 = query2.filter(
            models_proxy.UserResourcePermission.user_id.in_(user_ids))

    if not skip_group_perms and not skip_user_perms:
        query = query.union(query2)
    elif skip_group_perms:
        query = query2

    users = [PermissionTuple(row.User, row.perm_name, row.type,
                             row.Group or None, row.Resource, False, True)
             for row in query]
    return users


def permission_to_04_acls(permissions):
    """
    Legacy acl format kept for bw. compatibility
    :param permissions:
    :return:
    """
    acls = []
    for perm in permissions:
        if perm.type == 'user':
            acls.append((perm.user.id, perm.perm_name))
        elif perm.type == 'group':
            acls.append(('group:%s' % perm.group.id, perm.perm_name))
    return acls


def permission_to_pyramid_acls(permissions):
    """
    Returns a list of permissions in a format understood by pyramid
    :param permissions:
    :return:
    """
    acls = []
    for perm in permissions:
        if perm.type == 'user':
            acls.append((Allow, perm.user.id, perm.perm_name))
        elif perm.type == 'group':
            acls.append((Allow, 'group:%s' % perm.group.id, perm.perm_name))
    return acls



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import validates

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.services.group_resource_permission import \
    GroupResourcePermissionService

__all__ = ['GroupResourcePermissionMixin']


class GroupResourcePermissionMixin(BaseModel):
    """
    Mixin for GroupResourcePermission model
    """
    __table_args__ = (sa.PrimaryKeyConstraint(
        'group_id', 'resource_id', 'perm_name',
        name='pk_users_resources_permissions '),
                      {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'})

    _ziggurat_services = [GroupResourcePermissionService]

    @declared_attr
    def __tablename__(self):
        return 'groups_resources_permissions'

    @declared_attr
    def group_id(self):
        return sa.Column(sa.Integer, sa.ForeignKey('groups.id',
                                                   onupdate='CASCADE',
                                                   ondelete='CASCADE'),
                         primary_key=True)

    @declared_attr
    def resource_id(self):
        return sa.Column(sa.Integer(),
                         sa.ForeignKey('resources.resource_id',
                                       onupdate='CASCADE',
                                       ondelete='CASCADE'),
                         primary_key=True,
                         autoincrement=False)

    @declared_attr
    def perm_name(self):
        return sa.Column(sa.Unicode(50), primary_key=True)

    @validates('perm_name')
    def validate_perm_name(self, key, value):
        if value != value.lower():
            raise AssertionError('perm_name needs to be lowercase')
        return value

    def __repr__(self):
        return '<GroupResourcePermission: g:%s, %s, r:%s>' % (self.group_id,
                                                              self.perm_name,
                                                              self.resource_id,)



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import pytest

from ziggurat_foundations.models.services.group_permission import \
    GroupPermissionService
from ziggurat_foundations.models.services.group_resource_permission import \
    GroupResourcePermissionService
from ziggurat_foundations.models.services.user_permission import \
    UserPermissionService
from ziggurat_foundations.models.services.user_resource_permission import \
    UserResourcePermissionService

from ziggurat_foundations.models.services.resource import ResourceService

from ziggurat_foundations.permissions import PermissionTuple, ALL_PERMISSIONS
from ziggurat_foundations.tests import (
    add_user, check_one_in_other, add_resource, add_resource_b, add_group,
    BaseTestCase)
from ziggurat_foundations.tests.conftest import (
    User,
    UserPermission,
    GroupPermission,
    UserResourcePermission,
    GroupResourcePermission, TestResourceB)


class TestUserPermissions(BaseTestCase):
    def test_user_permissions(self, db_session):
        created_user = add_user(db_session)
        permissions = created_user.permissions
        expected = [
            PermissionTuple(created_user, 'alter_users', 'user', None, None,
                            False, True),
            PermissionTuple(created_user, 'root', 'user', None, None, False,
                            True)]
        check_one_in_other(permissions, expected)

    def test_owned_permissions(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        created_user.resources.append(resource)
        db_session.flush()
        resources = created_user.resources_with_perms(
            ['test_perm'], db_session=db_session).all()
        assert resources[0] == resource

    def test_resources_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(
            perm_name='test_perm', user_id=created_user.id,
            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      db_session=db_session).all()
        assert resources[0] == resource

    def test_mixed_perms(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(
            perm_name='test_perm', user_id=created_user.id,
            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        resource2 = add_resource(db_session, 2, 'test_resource')
        created_user.resources.append(resource2)
        resource3 = add_resource(db_session, 3, 'test_resource')
        resource4 = add_resource_b(db_session, 4, 'test_resource')
        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      db_session=db_session).all()
        found_ids = [r.resource_id for r in resources]
        assert sorted(found_ids) == [1, 2]

    def test_resources_with_perm_type_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(perm_name='test_perm',
                                            user_id=created_user.id,
                                            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      resource_types=[
                                                          'test_resource'],
                                                      db_session=db_session).all()
        assert resources[0] == resource

    def test_resources_with_perm_type_not_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(perm_name='test_perm',
                                            user_id=created_user.id,
                                            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      resource_types=[
                                                          'test_resource_b'],
                                                      db_session=db_session).all()
        assert resources == []

    def test_resources_with_perm_type_other_found(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        resource2 = add_resource_b(db_session, 2, 'test_resource')
        resource3 = add_resource(db_session, 3, 'test_resource')
        resource4 = add_resource_b(db_session, 4, 'test_resource')
        db_session.flush()
        permission = UserResourcePermission(perm_name='test_perm',
                                            user_id=created_user.id,
                                            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        permission2 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource2.resource_id)
        resource2.user_permissions.append(permission2)
        permission3 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource3.resource_id)
        resource3.user_permissions.append(permission3)
        permission4 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource4.resource_id)
        resource4.user_permissions.append(permission4)
        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      resource_types=[
                                                          'test_resource_b'],
                                                      db_session=db_session).all()
        assert len(resources) == 2

    def test_resources_with_wrong_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(
            perm_name='test_perm_bad',
            user_id=created_user.id,
            resource_id=resource.resource_id
        )
        with pytest.raises(AssertionError):
            resource.user_permissions.append(permission)

    def test_multiple_resources_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(perm_name='test_perm',
                                            user_id=created_user.id,
                                            resource_id=resource.resource_id
                                            )
        resource.user_permissions.append(permission)
        resource2 = add_resource(db_session, 2, 'test_resource2')
        permission2 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource2.resource_id
                                             )
        resource2.user_permissions.append(permission2)
        resources = created_user.resources_with_perms(['test_perm'],
                                                      db_session=db_session).all()
        assert resources == [resource, resource2]

    def test_resources_ids_with_perm(self, db_session):
        created_user = add_user(db_session)
        resource1 = add_resource(db_session, 1, 'test_resource1')
        resource2 = add_resource(db_session, 2, 'test_resource2')
        resource3 = add_resource(db_session, 3, 'test_resource3')

        permission1 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource1.resource_id)
        permission2 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource2.resource_id)
        permission3 = UserResourcePermission(perm_name='test_perm',
                                             user_id=created_user.id,
                                             resource_id=resource3.resource_id)

        resource1.user_permissions.append(permission1)
        resource2.user_permissions.append(permission2)
        resource3.user_permissions.append(permission3)

        db_session.flush()
        resources = created_user.resources_with_perms(['test_perm'],
                                                      resource_ids=[1, 3],
                                                      db_session=db_session).all()
        assert resources == [resource1, resource3]

    def test_resources_with_wrong_group_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        group = add_group(db_session, )
        group.users.append(created_user)
        group_permission = GroupResourcePermission(
            perm_name='test_perm_bad',
            group_id=group.id,
            resource_id=resource.resource_id
        )
        with pytest.raises(AssertionError):
            resource.group_permissions.append(group_permission)

    def test_resources_with_group_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        resource2 = add_resource(db_session, 2, 'test_resource2')
        add_resource(db_session, 3, 'test_resource3')
        group = add_group(db_session, )
        group.users.append(created_user)
        group_permission = GroupResourcePermission(
            perm_name='test_perm',
            group_id=1,
            resource_id=resource.resource_id
        )
        group_permission2 = GroupResourcePermission(
            perm_name='foo_perm',
            group_id=1,
            resource_id=resource2.resource_id
        )
        resource.group_permissions.append(group_permission)
        resource2.group_permissions.append(group_permission2)
        db_session.flush()
        resources = created_user.resources_with_perms(['foo_perm'],
                                                      db_session=db_session).all()
        assert resources[0] == resource2

    def test_resources_with_direct_user_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        # test_perm1 from group perms should be ignored
        perms = self.resource.direct_perms_for_user(
            self.user, db_session=db_session)
        second = [PermissionTuple(self.user, 'foo_perm', 'user', None,
                                  self.resource, False, True),
                  PermissionTuple(self.user, 'test_perm2', 'user', None,
                                  self.resource, False, True)]

        check_one_in_other(perms, second)

    def test_resources_with_direct_group_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        # test_perm1 from group perms should be ignored
        perms = self.resource.group_perms_for_user(
            self.user, db_session=db_session)
        second = [
            PermissionTuple(self.user, 'group_perm', 'group', self.group,
                            self.resource, False, True)]

        check_one_in_other(perms, second)

    def test_resources_with_user_perms(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.perms_for_user(
            self.user, db_session=db_session)
        second = [PermissionTuple(self.user, 'foo_perm', 'user', None,
                                  self.resource, False, True),
                  PermissionTuple(self.user, 'group_perm', 'group',
                                  self.group, self.resource, False, True),
                  PermissionTuple(self.user, 'test_perm2', 'user', None,
                                  self.resource, False, True)]

        check_one_in_other(perms, second)

    def test_resource_users_for_perm(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.users_for_perm(
            'foo_perm', db_session=db_session)
        second = [PermissionTuple(self.user, 'foo_perm', 'user', None,
                                  self.resource, False, True)]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.users_for_perm(
            '__any_permission__', db_session=db_session)
        second = [
            PermissionTuple(self.user, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user4, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_resource_2(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource2.users_for_perm(
            '__any_permission__', db_session=db_session)
        second = [
            PermissionTuple(self.user2, 'foo_perm', 'user', None,
                            self.resource2, False, True),
            PermissionTuple(self.user3, 'test_perm', 'user', None,
                            self.resource2, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_users(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.users_for_perm('__any_permission__',
                                             user_ids=[self.user.id],
                                             db_session=db_session)
        second = [
            PermissionTuple(self.user, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.users_for_perm('__any_permission__',
                                             user_ids=[self.user.id],
                                             group_ids=[self.group2.id],
                                             db_session=db_session)
        second = [
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_other_user_3(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource2.users_for_perm('__any_permission__',
                                              user_ids=[self.user3.id],
                                              db_session=db_session)
        second = [
            PermissionTuple(self.user3, 'test_perm', 'user', None,
                            self.resource2, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_other_user_4(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        perms = self.resource.users_for_perm('__any_permission__',
                                             user_ids=[self.user4.id],
                                             group_ids=[self.group2.id],
                                             db_session=db_session)
        second = [
            PermissionTuple(self.user4, 'group_perm', 'group', self.group2,
                            self.resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_users_limited_group_ownage(self, db_session):
        self.maxDiff = 9999
        self.set_up_user_group_and_perms(db_session)
        resource = TestResourceB(resource_id=99,
                                 resource_name='other',
                                 owner_user_id=self.user2.id)
        group3 = add_group(db_session, 'group 3')
        user2_permission = UserResourcePermission(
            perm_name='foo_perm',
            user_id=self.user2.id,
        )
        group3_permission = GroupResourcePermission(
            perm_name='group_perm',
            group_id=group3.id
        )
        resource.group_permissions.append(group3_permission)
        resource.user_permissions.append(user2_permission)
        group3.users.append(self.user3)
        self.user.resources.append(resource)
        self.group2.resources.append(resource)
        db_session.flush()
        perms = resource.users_for_perm('__any_permission__',
                                        db_session=db_session)
        second = [
            PermissionTuple(self.user2, 'foo_perm', 'user', None, resource,
                            False, True),
            PermissionTuple(self.user, ALL_PERMISSIONS, 'user', None, resource,
                            True, True),
            PermissionTuple(self.user4, ALL_PERMISSIONS, 'group', self.group2,
                            resource, True, True),
            PermissionTuple(self.user3, 'group_perm', 'group', group3,
                            resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_users_for_perms(self, db_session):
        user = User(user_name='aaa', email='aaa', status=0)
        user.set_password('password')
        aaa_perm = UserPermission(perm_name='aaa')
        bbb_perm = UserPermission(perm_name='bbb')
        bbb2_perm = UserPermission(perm_name='bbb')
        user.user_permissions.append(aaa_perm)
        user.user_permissions.append(bbb_perm)
        user2 = User(user_name='bbb', email='bbb', status=0)
        user2.set_password('password')
        user2.user_permissions.append(bbb2_perm)
        user3 = User(user_name='ccc', email='ccc', status=0)
        user3.set_password('password')
        group = add_group(db_session, )
        group.users.append(user3)
        db_session.add(user)
        db_session.add(user2)
        db_session.flush()
        users = User.users_for_perms(['aaa'], db_session=db_session)
        assert len(users.all()) == 1
        assert users[0].user_name == 'aaa'
        users = User.users_for_perms(['bbb'], db_session=db_session).all()
        assert len(users) == 2
        assert ['aaa', 'bbb'] == sorted([u.user_name for u in users])
        users = User.users_for_perms(['aaa', 'bbb', 'manage_apps'],
                                     db_session=db_session)
        assert ['aaa', 'bbb', 'ccc'] == sorted([u.user_name for u in users])

    def test_resources_with_possible_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        resource = TestResourceB(resource_id=3,
                                 resource_name='other',
                                 owner_user_id=self.user.id)
        self.user.resources.append(resource)
        resource_g = TestResourceB(resource_id=4,
                                   resource_name='group owned')
        self.group.resources.append(resource_g)
        db_session.flush()
        perms = self.user.resources_with_possible_perms()
        second = [PermissionTuple(self.user, 'foo_perm', 'user', None,
                                  self.resource, False, True),
                  PermissionTuple(self.user, 'group_perm', 'group',
                                  self.group, self.resource, False, True),
                  PermissionTuple(self.user, 'test_perm2', 'user', None,
                                  self.resource, False, True),
                  PermissionTuple(self.user, ALL_PERMISSIONS, 'user', None,
                                  resource, True, True),
                  PermissionTuple(self.user, ALL_PERMISSIONS, 'group',
                                  self.group, resource_g, True, True),
                  ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_additional_users(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = self.resource.users_for_perm(
            '__any_permission__', db_session=db_session)
        second = [
            PermissionTuple(self.user, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user6, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user7, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user6, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user7, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user4, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_limited_group_perms(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = self.resource.users_for_perm(
            '__any_permission__', limit_group_permissions=True,
            db_session=db_session)
        second = [
            PermissionTuple(None, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_additional_users(self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = self.resource.groups_for_perm(
            '__any_permission__', db_session=db_session)
        second = [
            PermissionTuple(self.user, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user6, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user7, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user6, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(user7, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user4, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_just_group_perms_limited(self,
                                                                   db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = self.resource.groups_for_perm(
            '__any_permission__', limit_group_permissions=True,
            db_session=db_session)
        second = [
            PermissionTuple(None, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_excluding_group_perms(
            self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        perms = self.resource.users_for_perm(
            '__any_permission__', limit_group_permissions=True,
            skip_group_perms=True, db_session=db_session)
        second = [
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_groups_for_any_perm_just_group_perms_limited_empty_group(
            self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)

        group3 = add_group(db_session, 'Empty group')
        perm3 = GroupResourcePermission(
            perm_name='group_permx',
            resource_id=self.resource.resource_id
        )
        group3.resource_permissions.append(perm3)
        perms = self.resource.groups_for_perm(
            '__any_permission__', limit_group_permissions=True,
            db_session=db_session)

        second = [
            PermissionTuple(None, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
            PermissionTuple(None, 'group_permx', 'group', group3,
                            self.resource, False, True)
        ]

        check_one_in_other(perms, second)

    def test_resource_users_for_any_perm_limited_group_perms_empty_group(
            self, db_session):
        self.maxDiff = 99999
        self.set_up_user_group_and_perms(db_session)
        user6 = add_user(db_session, 6, 'user 6')
        user7 = add_user(db_session, 7, 'user 7')
        perm2 = GroupResourcePermission(
            perm_name='group_perm2',
            resource_id=self.resource.resource_id
        )
        self.group.resource_permissions.append(perm2)
        self.group.users.append(user6)
        self.group.users.append(user7)
        group3 = add_group(db_session, 'Empty group')
        perm3 = GroupResourcePermission(
            perm_name='group_permx',
            resource_id=self.resource.resource_id
        )
        group3.resource_permissions.append(perm3)

        perms = self.resource.users_for_perm(
            '__any_permission__', limit_group_permissions=True,
            db_session=db_session)

        second = [
            PermissionTuple(None, 'group_perm', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm2', 'group', self.group,
                            self.resource, False, True),
            PermissionTuple(self.user, 'test_perm2', 'user', None,
                            self.resource, False, True),
            PermissionTuple(self.user, 'foo_perm', 'user', None,
                            self.resource, False, True),
            PermissionTuple(None, 'group_perm', 'group', self.group2,
                            self.resource, False, True),
            PermissionTuple(None, 'group_permx', 'group', group3,
                            self.resource, False, True),
        ]

        check_one_in_other(perms, second)

    def test_get_resource_permission(self, db_session):
        created_user = add_user(db_session)
        resource = add_resource(db_session, 1, 'test_resource')
        permission = UserResourcePermission(
            perm_name='test_perm', user_id=created_user.id,
            resource_id=resource.resource_id)
        resource.user_permissions.append(permission)
        db_session.flush()
        perm = UserResourcePermissionService.get(
            user_id=created_user.id,
            resource_id=resource.resource_id,
            perm_name='test_perm',
            db_session=db_session
        )
        assert perm.perm_name == 'test_perm'
        assert perm.resource_id == resource.resource_id
        assert perm.user_id == created_user.id


class TestGroupPermission(BaseTestCase):
    def test_repr(self, db_session):
        group_permission = GroupPermission(group_id=1,
                                           perm_name='perm')
        assert repr(group_permission) == '<GroupPermission: perm>'

    def test_get(self, db_session):
        org_group = add_group(db_session, 'group1')
        group = GroupPermissionService.get(
            group_id=org_group.id, perm_name='manage_apps',
            db_session=db_session)
        assert group.group_id == 1
        assert group.perm_name == 'manage_apps'

    def test_by_group_and_perm(self, db_session):
        add_group(db_session, )
        queried = GroupPermission.by_group_and_perm(1, 'manage_apps',
                                                    db_session=db_session)
        assert queried.group_id == 1
        assert queried.perm_name == 'manage_apps'

    def test_by_group_and_perm_wrong_group(self, db_session):
        add_group(db_session, )
        queried = GroupPermission.by_group_and_perm(2,
                                                    'manage_apps',
                                                    db_session=db_session)
        assert queried is None

    def test_by_group_and_perm_wrong_perm(self, db_session):
        add_group(db_session, )
        queried = GroupPermission.by_group_and_perm(1, 'wrong_perm',
                                                    db_session=db_session)
        assert queried is None

    def test_resources_with_possible_perms(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perms = self.group.resources_with_possible_perms()
        second = [PermissionTuple(None, 'group_perm', 'group', self.group,
                                  self.resource, False, True),
                  ]

        check_one_in_other(perms, second)

    def test_resources_with_possible_perms_group2(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        resource3 = add_resource_b(db_session, 3, 'other resource')
        self.group2.resources.append(resource3)
        group_permission2 = GroupResourcePermission(
            perm_name='group_perm2',
            group_id=self.group2.id,
        )
        self.resource2.group_permissions.append(group_permission2)

        perms = self.group2.resources_with_possible_perms()
        second = [PermissionTuple(None, 'group_perm', 'group', self.group2,
                                  self.resource, False, True),
                  PermissionTuple(None, 'group_perm2', 'group', self.group2,
                                  self.resource2, False, True),
                  PermissionTuple(None, ALL_PERMISSIONS, 'group', self.group2,
                                  resource3, True, True),
                  ]

        check_one_in_other(perms, second)

    def test_group_resource_permission(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        resource3 = add_resource_b(db_session, 3, 'other resource')
        db_session.flush()
        group_permission2 = GroupResourcePermission(
            perm_name='group_perm2',
            group_id=self.group2.id,
        )
        row = GroupResourcePermissionService.get(
            group_id=self.group2.id, resource_id=self.resource2.resource_id,
            perm_name='group_perm2', db_session=db_session)
        assert row is None
        self.resource2.group_permissions.append(group_permission2)
        row = GroupResourcePermissionService.get(
            group_id=self.group2.id, resource_id=self.resource2.resource_id,
            perm_name='group_perm2', db_session=db_session)
        assert row is not None

    def test_group_resource_permission_wrong(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perm_name = 'group_permX'
        perm = ResourceService.perm_by_group_and_perm_name(
            resource_id=self.resource.resource_id,
            group_id=self.group.id,
            perm_name=perm_name,
            db_session=db_session
        )
        assert perm is None

    def test_group_resource_permission(self, db_session):
        self.set_up_user_group_and_perms(db_session)
        perm_name = 'group_perm'
        perm = ResourceService.perm_by_group_and_perm_name(
            resource_id=self.resource.resource_id,
            group_id=self.group.id,
            perm_name=perm_name,
            db_session=db_session
        )
        assert perm.group_id == self.group.id
        assert perm.resource_id == self.resource.resource_id
        assert perm.perm_name == perm_name


class TestUserPermission(BaseTestCase):
    def test_repr(self, db_session):
        user_permission = UserPermission(user_id=1, perm_name='perm')
        assert repr(user_permission) == '<UserPermission: perm>'

    def test_get(self, db_session):
        user = add_user(db_session)
        perm = UserPermissionService.get(
            user_id=user.id, perm_name='root', db_session=db_session)
        assert perm.user_id == user.id
        assert perm.perm_name == 'root'

    def test_by_user_and_perm(self, db_session):
        add_user(db_session)
        user_permission = UserPermission.by_user_and_perm(1, 'root',
                                                          db_session=db_session)

        assert user_permission.user_id == 1
        assert user_permission.perm_name == 'root'

    def test_by_user_and_perm_wrong_username(self, db_session):
        add_user(db_session)
        user_permission = UserPermission.by_user_and_perm(999, 'root',
                                                          db_session=db_session)

        assert user_permission is None

    def test_by_user_and_perm_wrong_permname(self, db_session):
        add_user(db_session)
        user_permission = UserPermission.by_user_and_perm(1, 'wrong',
                                                          db_session=db_session)

        assert user_permission is None



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import os

import pytest

from ziggurat_foundations.models.services import BaseService
from ziggurat_foundations.tests import (
    BaseTestCase)
from ziggurat_foundations.tests.conftest import User

not_multi_db = os.environ.get("DB_STRING", '') == os.environ.get("DB_STRING2",
                                                                 '')


def add_user(i, db_name, db_session):
    user = User(id=i, user_name='username_{}_db_{}'.format(i, db_name),
                email='email_{}'.format(i),
                status=0)
    db_session.add(user)


@pytest.mark.skipif(not_multi_db, reason="requires different db strings")
class TestMultiDB(BaseTestCase):
    def test_users_in_two_databases(self, db_session, db_session2):
        for x in range(1, 5):
            add_user(x, 'a', db_session=db_session)
        for x in range(1, 3):
            add_user(x, 'b', db_session=db_session2)
        db_session.flush()
        db_session2.flush()

        db_users = BaseService.all(User, db_session=db_session).all()
        db2_users = BaseService.all(User, db_session=db_session2).all()
        assert len(db_users) == 4
        assert len(db2_users) == 2
        assert db_users[0].user_name == 'username_1_db_a'
        assert db2_users[0].user_name == 'username_1_db_b'



"""create indices on resource owners

Revision ID: 613e7c11dead
Revises: b5e6dd3449dd
Create Date: 2018-02-15 11:51:29.659352

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '613e7c11dead'
down_revision = 'b5e6dd3449dd'

from alembic import op

def upgrade():
    op.create_index(op.f('ix_resources_owner_group_id'), 'resources', ['owner_group_id'])
    op.create_index(op.f('ix_resources_owner_user_id'), 'resources', ['owner_user_id'])

def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__version__ = {'major': 0, 'minor': 7, 'patch': 2}


class ModelProxy(object):
    pass


class NOOP(object):
    def __nonzero__(self):
        return False

        # py3 compat

    __bool__ = __nonzero__


noop = NOOP()


def make_passwordmanager():
    from passlib.context import CryptContext
    pwd_context = CryptContext(
        # replace this list with the hash(es) you wish to support.
        # this example sets pbkdf2_sha256 as the default,
        # with support for legacy des_crypt hashes.
        schemes=["bcrypt", "pbkdf2_sha256"],
        default="bcrypt"
    )
    return pwd_context


def ziggurat_model_init(*args, **kwargs):
    """
    This function handles attaching model to service if model has one specified
    as `_ziggurat_service`, Also attached a proxy object holding all model
    definitions that services might use

    :param args:
    :param kwargs:
    :return:
    """
    models = ModelProxy()
    for cls2 in args:
        setattr(models, cls2.__name__, cls2)

    for cls in args:
        if cls.__name__ == 'User':
            if kwargs.get('passwordmanager'):
                cls.passwordmanager = kwargs['passwordmanager']
            else:
                cls.passwordmanager = make_passwordmanager()

        for cls2 in args:
            setattr(models, cls2.__name__, cls2)
        setattr(cls, "_ziggurat_models", models)
        # if model has a manager attached attached the class also to manager
        if hasattr(cls, '_ziggurat_services'):
            for service in cls._ziggurat_services:
                setattr(service, 'model', cls)
                setattr(service, 'models_proxy', models)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import validates
from zope.deprecation import deprecation

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.user_resource_permission import \
    UserResourcePermissionService

__all__ = ['UserResourcePermissionMixin']


class UserResourcePermissionMixin(BaseModel):
    """
    Mixin for UserResourcePermission model
    """
    __table_args__ = (sa.PrimaryKeyConstraint(
        'user_id', 'resource_id', 'perm_name',
        name='pk_users_resources_permissions '),
                      {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'})

    _ziggurat_services = [UserResourcePermissionService]

    @declared_attr
    def __tablename__(self):
        return 'users_resources_permissions'

    @declared_attr
    def user_id(self):
        return sa.Column(sa.Integer, sa.ForeignKey('users.id',
                                                   onupdate='CASCADE',
                                                   ondelete='CASCADE'),
                         primary_key=True)

    @declared_attr
    def resource_id(self):
        return sa.Column(sa.Integer(),
                         sa.ForeignKey('resources.resource_id',
                                       onupdate='CASCADE',
                                       ondelete='CASCADE'),
                         primary_key=True,
                         autoincrement=False)

    @declared_attr
    def perm_name(self):
        return sa.Column(sa.Unicode(64), primary_key=True)

    @validates('perm_name')
    def validate_perm_name(self, key, value):
        if value != value.lower():
            raise AssertionError('perm_name needs to be lowercase')
        return value

    def __repr__(self):
        return '<UserResourcePermission: %s, %s, %s>' % (self.user_id,
                                                         self.perm_name,
                                                         self.resource_id,)

    @classmethod
    @deprecation.deprecate("UserResourcePermission.by_resource_user_and_perm "
                           "will be removed in 0.8, use service instead")
    def by_resource_user_and_perm(cls, user_id, perm_name, resource_id,
                                  db_session=None):
        """

        .. deprecated:: 0.8

        :param user_id:
        :param perm_name:
        :param resource_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return UserResourcePermissionService.by_resource_user_and_perm(
            user_id=user_id, perm_name=perm_name, resource_id=resource_id,
            db_session=db_session)



"""add external identity tables

Revision ID: 24ab8d11f014
Revises: 2bb1ba973f0b
Create Date: 2011-11-10 23:18:19.446844

"""
from __future__ import unicode_literals

# downgrade revision identifier, used by Alembic.
revision = '24ab8d11f014'
down_revision = '2bb1ba973f0b'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('external_identities',
                    sa.Column('external_id', sa.Unicode(255), primary_key=True),
                    sa.Column('external_user_name', sa.Unicode(50), default=''),
                    sa.Column('local_user_name', sa.Unicode(50),
                              sa.ForeignKey('users.user_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('provider_name', sa.Unicode(50), default='',
                              primary_key=True),
                    sa.Column('access_token', sa.Unicode(255), default=''),
                    sa.Column('alt_token', sa.Unicode(255), default=''),
                    sa.Column('token_secret', sa.Unicode(255), default='')
                    )


def downgrade():
    drop_table('external_identities')



"""increase user and token field sized

Revision ID: b5e6dd3449dd
Revises: 57bbf0c387c
Create Date: 2016-05-05 11:00:59.915215

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = 'b5e6dd3449dd'
down_revision = '57bbf0c387c'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('external_identities', 'access_token',
                    type_=sa.Unicode(512), existing_type=sa.Unicode(255))
    op.alter_column('external_identities', 'alt_token',
                    type_=sa.Unicode(512), existing_type=sa.Unicode(255))
    op.alter_column('external_identities', 'token_secret',
                    type_=sa.Unicode(512), existing_type=sa.Unicode(255))
    op.alter_column('users', 'user_name',
                    type_=sa.Unicode(128), existing_type=sa.Unicode(32))
    op.alter_column('groups_permissions', 'perm_name',
                    type_=sa.Unicode(64), existing_type=sa.Unicode(30),
                    nullable=False)
    op.alter_column('groups_resources_permissions', 'perm_name',
                    type_=sa.Unicode(64), existing_type=sa.Unicode(50),
                    nullable=False)
    op.alter_column('users_permissions', 'perm_name',
                    type_=sa.Unicode(64), existing_type=sa.Unicode(30),
                    nullable=False)
    op.alter_column('users_resources_permissions', 'perm_name',
                    type_=sa.Unicode(64), existing_type=sa.Unicode(50),
                    nullable=False)


def downgrade():
    pass



"""initial table layout

Revision ID: 2bb1ba973f0b
Revises: None
Create Date: 2011-11-10 22:32:14.464939

"""
from __future__ import unicode_literals

# downgrade revision identifier, used by Alembic.
revision = '2bb1ba973f0b'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('groups',
                    sa.Column('id', sa.Integer, primary_key=True,
                              autoincrement=True),
                    sa.Column('group_name', sa.Unicode(50), unique=True),
                    sa.Column('description', sa.Text()),
                    sa.Column('member_count', sa.Integer, nullable=False,
                              default=0)
                    )
    op.create_table('groups_permissions',
                    sa.Column('group_name', sa.Unicode(50),
                              sa.ForeignKey('groups.group_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('perm_name', sa.Unicode(30), primary_key=True)
                    )

    op.create_table('users',
                    sa.Column('id', sa.Integer, primary_key=True,
                              autoincrement=True),
                    sa.Column('user_name', sa.Unicode(30), unique=True),
                    sa.Column('user_password', sa.Unicode(40)),
                    sa.Column('email', sa.Unicode(100), nullable=False,
                              unique=True),
                    sa.Column('status', sa.SmallInteger(), nullable=False),
                    sa.Column('security_code', sa.Unicode(40),
                              default='default'),
                    sa.Column('last_login_date', sa.TIMESTAMP(timezone=False),
                              default=sa.sql.func.now(),
                              server_default=sa.func.now()
                              )
                    )

    op.create_table('users_permissions',
                    sa.Column('user_name', sa.Unicode(50),
                              sa.ForeignKey('users.user_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('perm_name', sa.Unicode(30), primary_key=True)
                    )

    op.create_table('users_groups',
                    sa.Column('group_name', sa.Unicode(50),
                              sa.ForeignKey('groups.group_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('user_name', sa.Unicode(30),
                              sa.ForeignKey('users.user_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True)
                    )

    op.create_table('resources',
                    sa.Column('resource_id', sa.BigInteger(), primary_key=True,
                              nullable=False, autoincrement=True),
                    sa.Column('resource_name', sa.Unicode(100), nullable=False),
                    sa.Column('resource_type', sa.Unicode(30), nullable=False),
                    sa.Column('owner_group_name', sa.Unicode(50),
                              sa.ForeignKey('groups.group_name',
                                            onupdate='CASCADE',
                                            ondelete='SET NULL')),
                    sa.Column('owner_user_name', sa.Unicode(30),
                              sa.ForeignKey('users.user_name',
                                            onupdate='CASCADE',
                                            ondelete='SET NULL'))
                    )

    op.create_table('groups_resources_permissions',
                    sa.Column('group_name', sa.Unicode(50),
                              sa.ForeignKey('groups.group_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('resource_id', sa.BigInteger(),
                              sa.ForeignKey('resources.resource_id',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True,
                              autoincrement=False),
                    sa.Column('perm_name', sa.Unicode(50), primary_key=True)
                    )

    op.create_table('users_resources_permissions',
                    sa.Column('user_name', sa.Unicode(50),
                              sa.ForeignKey('users.user_name',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True),
                    sa.Column('resource_id', sa.BigInteger(),
                              sa.ForeignKey('resources.resource_id',
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                              primary_key=True,
                              autoincrement=False),
                    sa.Column('perm_name', sa.Unicode(50), primary_key=True)
                    )


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr

from ziggurat_foundations.models.base import BaseModel

__all__ = ['UserGroupMixin']


class UserGroupMixin(BaseModel):
    """
    Mixin for UserGroup model
    """
    __table_args__ = {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'}

    @declared_attr
    def __tablename__(self):
        return 'users_groups'

    @declared_attr
    def group_id(self):
        return sa.Column(sa.Integer,
                         sa.ForeignKey('groups.id', onupdate='CASCADE',
                                       ondelete='CASCADE'), primary_key=True)

    @declared_attr
    def user_id(self):
        return sa.Column(sa.Integer,
                         sa.ForeignKey('users.id', onupdate='CASCADE',
                                       ondelete='CASCADE'), primary_key=True)

    def __repr__(self):
        return '<UserGroup: g:%s, u:%s>' % (self.group_id, self.user_id,)



"""add local_user_id

Revision ID: 57bbf0c387c
Revises: 13391c68750
Create Date: 2016-04-02 09:54:09.643644

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '57bbf0c387c'
down_revision = '13391c68750'

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import table


def upgrade():
    conn = op.get_bind()
    op.add_column('external_identities', sa.Column('local_user_id',
                                                   sa.Integer()))

    external_identities_t = table('external_identities',
                                  sa.Column('local_user_name', sa.Unicode(50)),
                                  sa.Column('local_user_id', sa.Integer))
    users_t = table('users',
                    sa.Column('user_name', sa.Unicode(50)),
                    sa.Column('id', sa.Integer))

    stmt = external_identities_t.update().values(local_user_id=users_t.c.id). \
        where(users_t.c.user_name == external_identities_t.c.local_user_name)
    conn.execute(stmt)
    op.drop_constraint('pk_external_identities', 'external_identities',
                       type='primary')
    op.drop_constraint('fk_external_identities_local_user_name_users',
                       'external_identities', type='foreignkey')
    op.drop_column('external_identities', 'local_user_name')
    op.create_primary_key('pk_external_identities', 'external_identities',
                          cols=['external_id', 'local_user_id',
                                'provider_name'])
    op.create_foreign_key(None, 'external_identities', 'users',
                          remote_cols=['id'],
                          local_cols=['local_user_id'], onupdate='CASCADE',
                          ondelete='CASCADE')


def downgrade():
    pass



"""add security_code_date

Revision ID: 13391c68750
Revises: 438c27ec1c9
Create Date: 2015-06-14 21:28:30.672968

"""
from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '13391c68750'
down_revision = '438c27ec1c9'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('security_code_date', sa.DateTime(),
                                     server_default='2000-01-01 01:01'))


def downgrade():
    pass



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

import six

from ziggurat_foundations.models.services.user import UserService
from ziggurat_foundations.tests import BaseTestCase, add_user
from ziggurat_foundations.tests.conftest import (User)


class TestUser(BaseTestCase):
    def test_get(self, db_session):
        org_user = add_user(db_session)
        user = UserService.get(user_id=org_user.id, db_session=db_session)
        assert org_user.id == user.id

    def test_add_user(self, db_session):
        user = User(user_name='username', email='email', status=0)
        db_session.add(user)
        db_session.flush()

        user = db_session.query(User).filter(User.user_name == 'username')
        user = user.first()
        assert user.user_name == 'username'
        assert user.email == 'email'
        assert user.status == 0

    def test_delete_user(self, db_session):
        add_user(db_session)
        to_delete = User.by_user_name('username', db_session=db_session)
        db_session.delete(to_delete)
        db_session.commit()

    def test_user_repr(self, db_session):
        user = add_user(db_session)
        assert repr(user) == '<User: username>'

    def test_check_password_correct(self, db_session):
        user = add_user(db_session)
        assert user.check_password('password') is True

    def test_check_password_wrong(self, db_session):
        user = add_user(db_session)
        assert user.check_password('wrong_password') is False

    def test_by_user_name_existing(self, db_session):
        created_user = add_user(db_session)
        queried_user = User.by_user_name('username', db_session=db_session)

        assert created_user == queried_user

    def test_by_user_name_not_existing(self, db_session):
        add_user(db_session)
        queried_user = User.by_user_name('not_existing_user',
                                         db_session=db_session)

        assert queried_user is None

    def test_by_user_name_none(self, db_session):
        queried_user = User.by_user_name(None, db_session=db_session)

        assert queried_user is None

    def test_by_username_andsecurity_code_existing(self, db_session):
        created_user = add_user(db_session)
        security_code = created_user.security_code
        queried_user = User.by_user_name_and_security_code(
            user_name='username',
            security_code=security_code,
            db_session=db_session
        )

        assert created_user == queried_user

    def test_by_username_andsecurity_code_not_existing(self, db_session):
        created_user = add_user(db_session)
        security_code = created_user.security_code
        queried_user = User.by_user_name_and_security_code(
            user_name='not_existing_user',
            security_code=security_code,
            db_session=db_session
        )

        assert queried_user is None

    def test_by_username_andsecurity_code_wrong_code(self, db_session):
        add_user(db_session)
        queried_user = User.by_user_name_and_security_code(
            user_name='username',
            security_code='wrong_code',
            db_session=db_session
        )

        assert queried_user is None

    def test_by_username_andsecurity_code_none(self, db_session):
        created_user = add_user(db_session)
        security_code = created_user.security_code
        found = User.by_user_name_and_security_code(
            user_name=None,
            security_code=security_code,
            db_session=db_session
        )

        assert found is None

    def test_by_user_names(self, db_session):
        user1 = add_user(db_session, 'user1', 'email1')
        add_user(db_session, 'user2', 'email2')
        user3 = add_user(db_session, 'user3', 'email3')

        queried_users = User.by_user_names(['user1', 'user3'],
                                           db_session=db_session).all()

        assert len(queried_users) == 2
        assert user1 == queried_users[0]
        assert user3 == queried_users[1]

    def test_by_user_names_one_none(self, db_session):
        user1 = add_user(db_session, 'user1', 'email1')
        add_user(db_session, 'user2', 'email2')
        user3 = add_user(db_session, 'user3', 'email3')

        queried_users = User.by_user_names(['user1', None, 'user3'],
                                           db_session=db_session).all()

        assert len(queried_users) == 2
        assert user1 == queried_users[0]
        assert user3 == queried_users[1]

    def test_by_user_names_like(self, db_session):
        user1 = add_user(db_session, 'user1', 'email1')
        add_user(db_session, 'luser2', 'email2')
        add_user(db_session, 'noname', 'email3')

        queried_users = User.user_names_like('user%',
                                             db_session=db_session).all()
        assert len(queried_users) == 1
        assert user1 == queried_users[0]

    def test_by_user_names_like_none(self, db_session):
        queried_users = User.user_names_like(None,
                                             db_session=db_session).all()
        assert [] == queried_users

    def test_by_email(self, db_session):
        created_user = add_user(db_session)
        queried_user = User.by_email('email', db_session=db_session)

        assert created_user == queried_user

    def test_by_email_none(self, db_session):
        add_user(db_session)
        queried_user = User.by_email(None, db_session=db_session)

        assert queried_user is None

    def test_by_email_wrong_email(self, db_session):
        add_user(db_session)
        queried_user = User.by_email('wrong_email', db_session=db_session)

        assert queried_user is None

    def test_by_mail_and_username(self, db_session):
        created_user = add_user(db_session)
        queried_user = User.by_email_and_username('email', 'username',
                                                  db_session=db_session)

        assert created_user == queried_user

    def test_by_mail_and_username_wrong_mail(self, db_session):
        add_user(db_session)
        queried_user = User.by_email_and_username('wrong_email', 'username',
                                                  db_session=db_session)

        assert queried_user is None

    def test_by_mail_and_username_wrong_username(self, db_session):
        add_user(db_session)
        queried_user = User.by_email_and_username('email', 'wrong_username',
                                                  db_session=db_session)

        assert queried_user is None

    def test_by_mail_and_username_none(self, db_session):
        found = User.by_email_and_username('email', None,
                                           db_session=db_session)
        assert found is None

    def test_gravatar_url(self, db_session):
        user = add_user(db_session)
        user.email = 'arkadiy@bk.ru'
        assert user.gravatar_url() == 'https://secure.gravatar.com/avatar/' \
                                      'cbb6777e4a7ec0d96b33d2033e59fec6?d=mm'

    def test_gravatar_url_with_params(self, db_session):
        import six.moves.urllib.parse as parser
        user = add_user(db_session)
        user.email = 'arkadiy@bk.r'
        gravatar_url = user.gravatar_url(s=100, r='pg')
        parsed_url = parser.urlparse(gravatar_url)
        qs_dict = parser.parse_qs(parsed_url.query)
        assert qs_dict == {'s': ['100'], 'd': ['mm'], 'r': ['pg']}

    def test_generate_random_string(self, db_session):
        rand_str = User.generate_random_string()

        assert len(rand_str) == 7
        assert isinstance(rand_str, six.string_types)

    def test_generate_random_pass(self, db_session):
        rand_str = User.generate_random_pass()

        assert len(rand_str) == 7
        assert isinstance(rand_str, six.string_types)

        rand_str = User.generate_random_pass(20)
        assert len(rand_str) == 20

    def test_regenerate_security_code(self, db_session):
        created_user = add_user(db_session)
        old_code = created_user.security_code
        created_user.regenerate_security_code()
        new_code = created_user.security_code

        assert old_code != new_code
        assert len(new_code) == 64



import os

from setuptools import setup, find_packages

from ziggurat_foundations import __version__

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()
with open(os.path.join(here, 'CHANGES.rst')) as f:
    CHANGES = f.read()

version = '{}.{}.{}'.format(__version__['major'],
                            __version__['minor'],
                            __version__['patch'])

setup(
    name='ziggurat_foundations',
    version=version,
    description=""" Set of classes that are reusable across various types of
    web apps, base user object, auth relationships + structured resource tree
    """,
    long_description=README,
    author='Marcin Lulek',
    author_email='info@webreactor.eu',
    url='https://github.com/ergo/ziggurat_foundations',
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Flask',
        'Framework :: Pylons',
        'Framework :: Pyramid',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    packages=find_packages(),
    zip_safe=True,
    include_package_data=True,
    test_suite='ziggurat_foundations.tests',
    tests_require=[
        "coverage",
        "pytest",
        "psycopg2",
        "pytest-cov"
    ],
    install_requires=[
        "sqlalchemy",
        "passlib>=1.6.1",
        "paginate",
        "paginate_sqlalchemy",
        "alembic",
        'zope.deprecation >= 3.5.0',
        "six"]
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa

from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services import BaseService
from ziggurat_foundations.permissions import (
    ANY_PERMISSION,
    ALL_PERMISSIONS,
    PermissionTuple,
    resource_permissions_for_users)

__all__ = ['ResourceService']


class ResourceService(BaseService):
    @classmethod
    def get(cls, resource_id, db_session=None):
        """
        Fetch row using primary key -
        will use existing object in session if already present

        :param resource_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return db_session.query(cls.model).get(resource_id)

    @classmethod
    def perms_for_user(cls, instance, user, db_session=None):
        """
        returns all permissions that given user has for this resource
            from groups and directly set ones too

        :param instance:
        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, instance)
        query = db_session.query(
            cls.models_proxy.GroupResourcePermission.group_id.label(
                'owner_id'),
            cls.models_proxy.GroupResourcePermission.perm_name,
            sa.literal('group').label('type'))
        query = query.filter(
            cls.models_proxy.GroupResourcePermission.group_id.in_(
                [gr.id for gr in user.groups]
            )
        )
        query = query.filter(
            cls.models_proxy.GroupResourcePermission.resource_id ==
            instance.resource_id)

        query2 = db_session.query(
            cls.models_proxy.UserResourcePermission.user_id.label('owner_id'),
            cls.models_proxy.UserResourcePermission.perm_name,
            sa.literal('user').label('type'))
        query2 = query2.filter(
            cls.models_proxy.UserResourcePermission.user_id ==
            user.id)
        query2 = query2.filter(
            cls.models_proxy.UserResourcePermission.resource_id ==
            instance.resource_id)
        query = query.union(query2)

        groups_dict = dict([(g.id, g) for g in user.groups])
        perms = [PermissionTuple(user,
                                 row.perm_name,
                                 row.type,
                                 groups_dict.get(row.owner_id) if
                                 row.type == 'group' else None,
                                 instance, False, True) for row in query]

        # include all perms if user is the owner of this resource
        if instance.owner_user_id == user.id:
            perms.append(PermissionTuple(user, ALL_PERMISSIONS, 'user',
                                         None, instance, True, True))
        groups_dict = dict([(g.id, g) for g in user.groups])
        if instance.owner_group_id in groups_dict:
            perms.append(PermissionTuple(user, ALL_PERMISSIONS, 'group',
                                         groups_dict.get(
                                             instance.owner_group_id),
                                         instance, True, True))

        return perms

    @classmethod
    def direct_perms_for_user(cls, instance, user, db_session=None):
        """
        returns permissions that given user has for this resource
            without ones inherited from groups that user belongs to

        :param instance:
        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, instance)
        query = db_session.query(
            cls.models_proxy.UserResourcePermission.user_id,
            cls.models_proxy.UserResourcePermission.perm_name)
        query = query.filter(cls.models_proxy.UserResourcePermission.user_id ==
                             user.id)
        query = query.filter(
            cls.models_proxy.UserResourcePermission.resource_id ==
            instance.resource_id)

        perms = [PermissionTuple(user,
                                 row.perm_name,
                                 'user',
                                 None,
                                 instance, False, True) for row in query]

        # include all perms if user is the owner of this resource
        if instance.owner_user_id == user.id:
            perms.append(PermissionTuple(user, ALL_PERMISSIONS, 'user',
                                         None, instance, True))
        return perms

    @classmethod
    def group_perms_for_user(cls, instance, user, db_session=None):
        """
        returns permissions that given user has for this resource
            that are inherited from groups

        :param instance:
        :param user:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, instance)
        perms = resource_permissions_for_users(cls.models_proxy,
                                               ANY_PERMISSION,
                                               resource_ids=[
                                                   instance.resource_id],
                                               user_ids=[user.id],
                                               db_session=db_session)
        perms = [p for p in perms if p.type == 'group']
        # include all perms if user is the owner of this resource
        groups_dict = dict([(g.id, g) for g in user.groups])
        if instance.owner_group_id in groups_dict:
            perms.append(PermissionTuple(user, ALL_PERMISSIONS, 'group',
                                         groups_dict.get(
                                             instance.owner_group_id),
                                         instance, True, True))
        return perms

    @classmethod
    def users_for_perm(cls, instance, perm_name, user_ids=None, group_ids=None,
                       limit_group_permissions=False, skip_group_perms=False,
                       db_session=None):
        """
        return PermissionTuples for users AND groups that have given
        permission for the resource, perm_name is __any_permission__ then
        users with any permission will be listed

        :param instance:
        :param perm_name:
        :param user_ids: limits the permissions to specific user ids
        :param group_ids: limits the permissions to specific group ids
        :param limit_group_permissions: should be used if we do not want to have
        user objects returned for group permissions, this might cause performance
        issues for big groups
        :param skip_group_perms: do not attach group permissions to the resultset
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, instance)
        users_perms = resource_permissions_for_users(cls.models_proxy,
                                                     [perm_name],
                                                     [instance.resource_id],
                                                     user_ids=user_ids,
                                                     group_ids=group_ids,
                                                     limit_group_permissions=limit_group_permissions,
                                                     skip_group_perms=skip_group_perms,
                                                     db_session=db_session)
        if instance.owner_user_id:
            users_perms.append(
                PermissionTuple(instance.owner,
                                ALL_PERMISSIONS, 'user', None, instance, True,
                                True))
        if instance.owner_group_id and not skip_group_perms:
            for user in instance.owner_group.users:
                users_perms.append(
                    PermissionTuple(user, ALL_PERMISSIONS, 'group',
                                    instance.owner_group, instance, True,
                                    True))

        return users_perms

    @classmethod
    def by_resource_id(cls, resource_id, db_session=None):
        """
        fetch the resouce by id

        :param resource_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.model).filter(cls.model.resource_id ==
                                                   int(resource_id))
        return query.first()

    @classmethod
    def perm_by_group_and_perm_name(cls, resource_id, group_id, perm_name,
                                    db_session=None):
        """
        fetch permissions by group and permission name

        :param resource_id:
        :param group_id:
        :param perm_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.models_proxy.GroupResourcePermission)
        query = query.filter(
            cls.models_proxy.GroupResourcePermission.group_id == group_id)
        query = query.filter(
            cls.models_proxy.GroupResourcePermission.perm_name == perm_name)
        query = query.filter(
            cls.models_proxy.GroupResourcePermission.resource_id == resource_id)
        return query.first()

    @classmethod
    def groups_for_perm(cls, instance, perm_name, group_ids=None,
                        limit_group_permissions=False,
                        db_session=None):
        """
        return PermissionTuples for groups that have given
        permission for the resource, perm_name is __any_permission__ then
        users with any permission will be listed

        :param instance:
        :param perm_name:
        :param group_ids: limits the permissions to specific group ids
        :param limit_group_permissions: should be used if we do not want to have
        user objects returned for group permissions, this might cause performance
        issues for big groups
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, instance)
        group_perms = resource_permissions_for_users(cls.models_proxy,
                                                     [perm_name],
                                                     [instance.resource_id],
                                                     group_ids=group_ids,
                                                     limit_group_permissions=limit_group_permissions,
                                                     skip_user_perms=True,
                                                     db_session=db_session)
        if instance.owner_group_id:
            for user in instance.owner_group.users:
                group_perms.append(
                    PermissionTuple(user, ALL_PERMISSIONS, 'group',
                                    instance.owner_group, instance, True,
                                    True))

        return group_perms

    @classmethod
    def lock_resource_for_update(cls, resource_id, db_session):
        """
        Selects resource for update - locking access for other transactions

        :param resource_id:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        query = db_session.query(cls.model)
        query = query.filter(cls.model.resource_id == resource_id)
        query = query.with_for_update()
        return query.first()



# -*- coding: utf-8 -*-
from __future__ import with_statement, unicode_literals

from ziggurat_foundations.tests.conftest import (
    User, Group, UserGroup, GroupPermission, UserPermission,
    UserResourcePermission, GroupResourcePermission, Resource,
    ExternalIdentity, TestResource, TestResourceB)


def check_one_in_other(first, second):
    while first:
        item = first.pop()
        ix = second.index(item)
        second.pop(ix)
    assert len(first) == len(second)


def add_user(db_session, user_name='username', email='email',
             perms=['root', 'alter_users']):
    user = User(user_name=user_name, email=email, status=0)
    user.set_password('password')
    for perm in perms:
        u_perm = UserPermission(perm_name=perm)
        user.user_permissions.append(u_perm)
    db_session.add(user)
    db_session.flush()
    return user


def add_resource(db_session, resource_id, resource_name='test_resource',
                 parent_id=None, ordering=None):
    Resource.__possible_permissions__ = [
        'test_perm', 'test_perm1',
        'test_perm2', 'foo_perm',
        'group_perm', 'group_perm2']
    resource = TestResource(resource_id=resource_id,
                            resource_name=resource_name,
                            parent_id=parent_id,
                            ordering=ordering)
    db_session.add(resource)
    db_session.flush()
    return resource


def add_resource_b(db_session, resource_id, resource_name='test_resource'):
    Resource.__possible_permissions__ = ['test_perm', 'test_perm1',
                                         'test_perm2', 'foo_perm',
                                         'group_perm', 'group_perm2']
    resource = TestResourceB(resource_id=resource_id,
                             resource_name=resource_name)
    db_session.add(resource)
    db_session.flush()
    return resource


def add_group(db_session, group_name='group', description='desc'):
    group = Group(
        group_name=group_name,
        description=description
    )
    test_perm = GroupPermission(perm_name='manage_apps')
    group.permissions.append(test_perm)
    db_session.add(group)
    db_session.flush()
    return group


def create_default_tree(db_session):
    root = add_resource(
        db_session, -1, 'root a', ordering=1)
    res_a = add_resource(
        db_session, 1, 'a', parent_id=root.resource_id, ordering=1)
    res_aa = add_resource(
        db_session, 5, 'aa', parent_id=res_a.resource_id, ordering=1)
    res_ab = add_resource(
        db_session, 6, 'ab', parent_id=res_a.resource_id, ordering=2)
    res_ac = add_resource(
        db_session, 7, 'ac', parent_id=res_a.resource_id, ordering=3)
    res_aca = add_resource(
        db_session, 9, 'aca', parent_id=res_ac.resource_id, ordering=1)
    res_acaa = add_resource(
        db_session, 12, 'acaa', parent_id=res_aca.resource_id, ordering=1)
    res_ad = add_resource(
        db_session, 8, 'ad', parent_id=res_a.resource_id, ordering=4)
    res_b = add_resource(
        db_session, 2, 'b', parent_id=root.resource_id, ordering=2)
    res_ba = add_resource(
        db_session, 4, 'ba', parent_id=res_b.resource_id, ordering=1)
    res_c = add_resource(
        db_session, 3, 'c', parent_id=root.resource_id, ordering=3)
    res_d = add_resource(
        db_session, 10, 'd', parent_id=root.resource_id, ordering=4)
    res_e = add_resource(
        db_session, 11, 'e', parent_id=root.resource_id, ordering=5)
    root_b = add_resource(
        db_session, -2, 'root b', ordering=2)
    root_c = add_resource(
        db_session, -3, 'root c', ordering=3)
    return [root, root_b, root_c]


class BaseTestCase(object):
    def set_up_user_group_and_perms(self, db_session):
        """
        perm map:

        username:
            first_user : root, alter_users
            res_perms: r1:g1:foo_perm, r1:g1:test_perm2

        foouser:
            user_perms : custom
            res_perms: r2:foo_perm

        baruser:
            user_perms : root, alter_users
            res_perms: r2:test_perm

        bazuser:
            user_perms : root, alter_users
            res_perms: r1:g2:group_perm

        """
        created_user = add_user(db_session, user_name="first_user")
        created_user2 = add_user(db_session, user_name='foouser',
                                 email='new_email',
                                 perms=['custom'])
        created_user3 = add_user(db_session,
                                 user_name='baruser', email='new_email2')
        created_user4 = add_user(db_session,
                                 user_name='bazuser', email='new_email3')
        resource = add_resource(db_session, 1, 'test_resource')
        resource2 = add_resource_b(db_session, 2, 'other_resource')
        group = add_group(db_session, )
        group2 = add_group(db_session, group_name='group2')
        group.users.append(created_user)
        group2.users.append(created_user4)
        group_permission = GroupResourcePermission(
            perm_name='group_perm',
            group_id=group.id,
        )
        group_permission2 = GroupResourcePermission(
            perm_name='group_perm',
            group_id=group2.id,
        )
        user_permission = UserResourcePermission(
            perm_name='test_perm2',
            user_id=created_user.id,
        )
        user_permission2 = UserResourcePermission(
            perm_name='foo_perm',
            user_id=created_user.id,
        )
        user2_permission = UserResourcePermission(
            perm_name='foo_perm',
            user_id=created_user2.id,
        )
        user3_permission = UserResourcePermission(
            perm_name='test_perm',
            user_id=created_user3.id,
        )
        resource.group_permissions.append(group_permission)
        resource.group_permissions.append(group_permission2)
        resource.user_permissions.append(user_permission)
        resource.user_permissions.append(user_permission2)
        resource2.user_permissions.append(user2_permission)
        resource2.user_permissions.append(user3_permission)
        db_session.flush()
        self.resource = resource
        self.resource2 = resource2
        self.user = created_user
        self.user2 = created_user2
        self.user3 = created_user3
        self.user4 = created_user4
        self.group = group
        self.group2 = group2


class DummyUserObj(object):
    def __init__(self):
        self.user_name = 'new_name'
        self.user_password = 'foo'
        self.email = 'change@email.com'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import importlib
import logging

from ziggurat_foundations.exc import ZigguratException
from ziggurat_foundations.models.base import get_db_session

CONFIG_KEY = 'ziggurat_foundations'
log = logging.getLogger(__name__)


def includeme(config):
    settings = config.registry.settings

    user_model_location = settings.get('%s.model_locations.User' % CONFIG_KEY)
    session_provider_callable = settings.get(
        '%s.session_provider_callable' % CONFIG_KEY)

    if not user_model_location:
        raise ZigguratException('''You need to pass location of user model
        inside your application eg.:
        ziggurat_foundations.model_locations.User = youappname.models:User
        ''')

    if not session_provider_callable:
        def session_provider_callable(request):
            return get_db_session()

        test_session_callable = None
    else:
        parts = session_provider_callable.split(':')
        _tmp = importlib.import_module(parts[0])
        session_provider_callable = getattr(_tmp, parts[1])
        test_session_callable = "session exists"

    parts = user_model_location.split(':')
    _tmp = importlib.import_module(parts[0])
    UserModel = getattr(_tmp, parts[1])

    # This function is bundled into the request, so for each request you can
    # do request.user
    def get_user(request):
        userid = request.unauthenticated_userid
        if test_session_callable == None:
            # set db_session to none to pass to the UserModel.by_id
            db_session = None
        else:
            # Else assign the request.session
            db_session = session_provider_callable(request)
        if userid is not None:
            return UserModel.by_id(userid, db_session=db_session)

    # add in request.user function
    config.set_request_property(get_user, 'user', reify=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from zope.deprecation import deprecation

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.group import GroupService

__all__ = ['GroupMixin']


class GroupMixin(BaseModel):
    """ Mixin for Group model """

    _ziggurat_services = [GroupService]

    __table_args__ = {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'}

    @declared_attr
    def __tablename__(self):
        return 'groups'

    # lists app wide permissions we might want to assign to groups
    __possible_permissions__ = ()

    @declared_attr
    def id(self):
        return sa.Column(sa.Integer(), primary_key=True, )

    @declared_attr
    def group_name(self):
        return sa.Column(sa.Unicode(128), nullable=False, unique=True)

    @declared_attr
    def description(self):
        return sa.Column(sa.Text())

    @declared_attr
    def member_count(self):
        return sa.Column(sa.Integer, nullable=False, default=0)

    @declared_attr
    def users(self):
        """ relationship for users belonging to this group"""
        return sa.orm.relationship('User',
                                   secondary='users_groups',
                                   order_by='User.user_name',
                                   passive_deletes=True,
                                   passive_updates=True,
                                   backref='groups')

    # dynamic property - useful
    @declared_attr
    def users_dynamic(self):
        """ dynamic relationship for users belonging to this group
            one can use filter """
        return sa.orm.relationship('User',
                                   secondary='users_groups',
                                   order_by='User.user_name',
                                   lazy="dynamic")

    @declared_attr
    def permissions(self):
        """ non-resource permissions assigned to this group"""
        return sa.orm.relationship('GroupPermission',
                                   backref='groups',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def resource_permissions(self):
        """ permissions to specific resources this group has"""
        return sa.orm.relationship('GroupResourcePermission',
                                   backref='groups',
                                   cascade="all, delete-orphan",
                                   passive_deletes=True,
                                   passive_updates=True)

    @declared_attr
    def resources(cls):
        """ Returns all resources directly owned by group, can be used to assign
        ownership of new resources::

            user.resources.append(resource) """
        return sa.orm.relationship('Resource',
                                   cascade="all",
                                   passive_deletes=True,
                                   passive_updates=True,
                                   backref='owner_group')

    @declared_attr
    def resources_dynamic(cls):
        """ Returns all resources directly owned by group, can be used to assign
        ownership of new resources::

            user.resources.append(resource) """
        return sa.orm.relationship('Resource',
                                   cascade="all",
                                   passive_deletes=True,
                                   passive_updates=True,
                                   lazy='dynamic')

    @sa.orm.validates('permissions')
    def validate_permission(self, key, permission):
        """ validates if group can get assigned with permission"""
        if permission.perm_name not in self.__possible_permissions__:
            raise AssertionError('perm_name is not one of {}'.format(
                self.__possible_permissions__))
        return permission

    def __repr__(self):
        return '<Group: %s, %s>' % (self.group_name, self.id)

    @classmethod
    @deprecation.deprecate("Group.by_group_name "
                           "will be removed in 0.8, use service instead")
    def by_group_name(cls, group_name, db_session=None):
        """ fetch group by name

        .. deprecated:: 0.8

        """
        db_session = get_db_session(db_session)
        return GroupService.by_group_name(group_name=group_name,
                                          db_session=db_session)

    @deprecation.deprecate("Group.get_user_paginator "
                           "will be removed in 0.8, use service instead")
    def get_user_paginator(self, page=1, item_count=None, items_per_page=50,
                           user_ids=None, GET_params=None, db_session=None):
        """

        .. deprecated:: 0.8

        :param page:
        :param item_count:
        :param items_per_page:
        :param user_ids:
        :param GET_params:
        :param db_session:
        :return:
        """
        return GroupService.get_user_paginator(
            self, page=page, item_count=item_count,
            items_per_page=items_per_page,
            user_ids=user_ids, GET_params=GET_params)

    @deprecation.deprecate("Group.resources_with_possible_perms "
                           "will be removed in 0.8, use service instead")
    def resources_with_possible_perms(self, perm_names=None, resource_ids=None,
                                      resource_types=None,
                                      db_session=None):
        """

        .. deprecated:: 0.8

        :param perm_names:
        :param resource_ids:
        :param resource_types:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session, self)
        return GroupService.resources_with_possible_perms(
            self, perm_names=perm_names, resource_ids=resource_ids,
            resource_types=resource_types, db_session=db_session)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import validates
from zope.deprecation import deprecation

from ziggurat_foundations.models.base import BaseModel
from ziggurat_foundations.models.base import get_db_session
from ziggurat_foundations.models.services.group_permission import \
    GroupPermissionService

__all__ = ['GroupPermissionMixin']


class GroupPermissionMixin(BaseModel):
    """ Mixin for GroupPermission model"""

    __table_args__ = (sa.PrimaryKeyConstraint('group_id', 'perm_name',
                                              name='pk_groups_permissions'),
                      {'mysql_engine': 'InnoDB', 'mysql_charset': 'utf8'})

    _ziggurat_services = [GroupPermissionService]

    @declared_attr
    def __tablename__(self):
        return 'groups_permissions'

    @declared_attr
    def group_id(self):
        return sa.Column(sa.Integer(),
                         sa.ForeignKey('groups.id', onupdate='CASCADE',
                                       ondelete='CASCADE'), primary_key=True)

    @declared_attr
    def perm_name(self):
        return sa.Column(sa.Unicode(64), primary_key=True)

    @validates('perm_name')
    def validate_perm_name(self, key, value):
        if value != value.lower():
            raise AssertionError('perm_name needs to be lowercase')
        return value

    def __repr__(self):
        return '<GroupPermission: %s>' % self.perm_name

    @classmethod
    @deprecation.deprecate("GroupPermission.by_group_and_perm "
                           "will be removed in 0.8, use service instead")
    def by_group_and_perm(cls, group_id, perm_name, db_session=None):
        """

        .. deprecated:: 0.8

        :param group_id:
        :param perm_name:
        :param db_session:
        :return:
        """
        db_session = get_db_session(db_session)
        return GroupPermissionService.by_group_and_perm(
            group_id=group_id,
            perm_name=perm_name,
            db_session=db_session)
